Package: MFSD
Type: Package
Title: Multivariate Functional Spatial Data
Version: 0.1.1
Authors@R: 
    c(
      person("SI-AHMED","IDRIS",
             role=c("aut","cre"),
             email="i_siahmed@esi.dz"),
      person("Thomas","Kuenzer",
             role="ctb",
             comment="Code adapted from the fsd package"),
      person("Judith AGONKOUI","Christelle",
             role="ctb",
             email="christelle.agonkoui@imsp-uac.org"),
      person("HAMDAD","Leila",
             role="ctb",
             email="l_hamdad@esi.dz"),
      person("DABO-NIANG","Sophie",
             role="ctb",
             email="sophie.dabo@univ-lille.fr")
    )
Description: Analysis of multivariate functional spatial data, 
             including spectral multivariate functional principal 
             component analysis and related statistical procedures 
             (Si-Ahmed, Idris, et al. "Principal component analysis of multivariate spatial functional data." Big Data Research 39 (2025) 100504).
             (Kuenzer, T., Hörmann, S., & Kokoszka, P. (2021). "Principal component analysis of spatially indexed functions." Journal of the American Statistical Association, 116(535), 1444-1456.)
             (Happ, C., & Greven, S. (2018). "Multivariate functional principal component analysis for data observed on different (dimensional) domains." Journal of the American Statistical Association, 113(522), 649-659.)
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.3
Imports: fda, graphics, mvtnorm, Rcpp (>= 1.0.0)
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 2.10)
LinkingTo: Rcpp, RcppArmadillo
NeedsCompilation: yes
Packaged: 2026-04-18 18:30:00 UTC; User
Author: SI-AHMED IDRIS [aut, cre],
  Thomas Kuenzer [ctb] (Code adapted from the fsd package),
  Judith AGONKOUI Christelle [ctb],
  HAMDAD Leila [ctb],
  DABO-NIANG Sophie [ctb]
Maintainer: SI-AHMED IDRIS <i_siahmed@esi.dz>
Repository: CRAN
Date/Publication: 2026-04-20 23:20:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-21 19:20:31 UTC; unix
Archs: MFSD.so.dSYM

S3method("%/%",fsd.fd)
S3method("*",fsd.fd)
S3method("+",fsd.fd)
S3method("-",fsd.fd)
S3method("/",fsd.fd)
S3method("[",fsd.fd)
S3method(as.fd,fsd.fd)
S3method(diff,fsd.fd)
S3method(mean,fsd.fd)
S3method(plot,fsd.fd)
export(plot.fsd.fd)
export(as.fsd.fd)
export(center.fsd.fd)
export(fsd.covariance)
export(fsd.fd)
export(fsd.filter)
export(fsd.filter.is.unilateral)
export(fsd.jb.test)
export(fsd.norm)
export(fsd.perm)
export(fsd.plot.covariance)
export(fsd.plot.data)
export(fsd.plot.filters)
export(fsd.sfarma)
export(fsd.spca)
export(fsd.spca.cov)
export(fsd.spca.filters)
export(fsd.spca.inverse)
export(fsd.spca.scores)
export(fsd.spca.var)
export(fsd.spectral.density)
export(mfsd)
export(mfsd.evaluation)
export(mfsd_Filter)
export(mfsd_Scores)
export(unfold)
import(fda)
import(mvtnorm)
import(stats)
import(Rcpp)
importFrom(Rcpp,evalCpp)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,filled.contour)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,segments)
useDynLib(MFSD)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# MFSD

<!-- badges: start -->
<!-- badges: end -->

We present a novel approach called Spatial Multivariate Functional
Principal Component Analysis (SMFPCA), specifically designed for
analyzing spatially multivariate data. This package focuses on reducing
the dimensions of datasets with multiple variables. The data handled by
this package are spatially indexed and exhibit weak stationarity. Unlike
the traditional Multivariate Karhunen-Loève method, SMFPCA efficiently
captures spatial relationships within multiple functions. By performing
spectral functional component analysis on multivariate spatial data,
SMFPCA analyzes data points distributed over a regular grid.

## Installation

Install Rtools if not already installed. It can be downloaded from
<https://cran.r-project.org/bin/windows/Rtools/> The development version
of MFSD can be installed as follows:

``` r
if (!require(devtools)) install.packages("devtools")
require(devtools)
if (!find_rtools()) stop("Please install Rtools first.")
install_github("kuenzer/fsd")
```

## Example

converting each NOAA study variable into functional spatial data using
the fsd.fd function. In our context, we opted for variables delineating
the progression of sea surface temperature for the years 1999 and 2000.
Following the application of the fsd.fd function, we acquired functional
spatial variables designated as X1999 and X2000, respectively. The two
functional spatial variables are available in the MFSD package, and can
be loaded using the following R code:

``` r
library(MFSD)
#> Registered S3 methods overwritten by 'MFSD':
#>   method       from
#>   %/%.fsd.fd   fsd 
#>   *.fsd.fd     fsd 
#>   +.fsd.fd     fsd 
#>   -.fsd.fd     fsd 
#>   /.fsd.fd     fsd 
#>   [.fsd.fd     fsd 
#>   as.fd.fsd.fd fsd 
#>   diff.fsd.fd  fsd 
#>   mean.fsd.fd  fsd 
#>   plot.fsd.fd  fsd
data("X1999")
data("X2000")
```

We load the ‘Lliste’ variables from the MFSD package, which include two
variables indicating the maximum lag for the filters applied to X1999
and X2000. Additionally, we load ‘qlist’, which contains tuning
parameters for estimating the spectral density of X1999 and X2000
variables.

``` r
data("qlist")
data("Llist")
```

We structure the variables in the X, q and L lists, then we apply the
mfsd() function with the SM_Npc parameter set to 4. Then we perform
multivariate spatial FPCA on the years 1999 (X00\[,,17\]) and 2000
(X00\[,,18\]) :

``` r
X = list(X1999, X2000)
q = list(qlist[,17], qlist[,18])
L = list(L1 = Llist[17], L2 = Llist[18])

Xspecc <- mfsd(X = X, L = L, q = q, SM_Npc = 4)
#> Warning in fsd.spca.filters(F, Npc, L): The chosen clipping parameter L is low.
#> Summed squared norms for the PC filters are only 0.95, 0.87, 0.794, 0.758, 0.711, 0.718, 0.701, 0.697, 0.693, 0.695, 0.717, 0.735, 0.769, 0.779, 0.789
#> Warning in fsd.spca.filters(F, Npc, L): The chosen clipping parameter L is low.
#> Summed squared norms for the PC filters are only 0.915, 0.842, 0.769, 0.714, 0.686, 0.696, 0.642, 0.63, 0.634, 0.646, 0.656, 0.7, 0.728, 0.751, 0.851
Xspec <- mfsd(X = X, L = L, q = 0, SM_Npc = 4)
```

The variable Xspec, with the q parameter set to zero, indicates that the
mfsd function does not consider the spatial aspect, compared with the
Xspecc variable.

To visualize the evolution of the four functional spatial filters of the
variable Xspec, we use the function plot.fsd.filter.

``` r
fsd.plot.filters(Xspec[["filter"]][[1]], Npc = 4, Lmax = 2)
```

<img src="man/figures/README-unnamed-chunk-5-1.png" width="100%" />

``` r
fsd.plot.filters(Xspecc[["filter"]][[1]], Npc = 4, Lmax = 2)
```

<img src="man/figures/README-unnamed-chunk-6-1.png" width="100%" />

The following R code shows the evolution of the four fonctional spatial
filters of the variable Xpecc.

``` r
fsd.plot.filters(Xspec[["filter"]][[2]], Npc = 4, Lmax = 2)
```

<img src="man/figures/README-unnamed-chunk-7-1.png" width="100%" />

``` r
fsd.plot.filters(Xspecc[["filter"]][[2]], Npc = 4, Lmax = 2)
```

<img src="man/figures/README-unnamed-chunk-8-1.png" width="100%" />

We assess the effectiveness of dimensionality reduction by computing the
NMSE and NMSE\* for univariate components with the function
mfsd.evaluation.

``` r
X00    <- list(X1999, X2000)
result = mfsd.evaluation(Xspec, Xspecc, X00, Npc = 4 )
#>                  PC 1      PC 2      PC 3       PC 4
#> ADVANTAGE  0.06239868 0.1004192 0.1042668 0.09948148
#> NMSE_spat  0.35232024 0.2818504 0.2024973 0.16270440
#> NMSE_ord   0.41471892 0.3822696 0.3067641 0.26218589
#> NMSE_spat* 0.42540660 0.2739412 0.1889977 0.13191276
#> NMSE_ord*  0.47445535 0.3520799 0.2778462 0.21232158
#>                  PC 1       PC 2      PC 3      PC 4
#> ADVANTAGE  0.04920428 0.06739567 0.1614425 0.1065269
#> NMSE_spat  0.54606980 0.37840956 0.2655430 0.2047076
#> NMSE_ord   0.59527409 0.44580523 0.4269855 0.3112345
#> NMSE_spat* 0.43564017 0.25969140 0.1664391 0.1120781
#> NMSE_ord*  0.51567289 0.33421271 0.2695860 0.2086840
```

The results are shown in Table 1.

``` r
result
#> NULL
```

The results shown also details of explained cumulative variance of FPCA
after applying SMFPCA to TMP-1999 and TMP-2000.

``` r
Xspecc[["vm"]]
#>  [1]  58.85594  71.06221  81.25402  87.34009  91.01089  93.46554  95.15372
#>  [8]  96.23395  97.22491  97.93748  98.39807  98.71391  98.99927  99.20915
#> [15]  99.36222  99.49048  99.57933  99.66158  99.72081  99.77526  99.81341
#> [22]  99.84942  99.88515  99.91066  99.93498  99.95432  99.96818  99.98092
#> [29]  99.99196 100.00000
Xspec[["vm"]]
#>  [1]  46.96172  57.30220  64.63431  69.02982  73.07528  76.99652  80.11821
#>  [8]  82.65983  84.77960  86.60027  88.16848  89.60531  90.97761  92.16197
#> [15]  93.24352  94.23263  95.16237  95.89696  96.54831  97.14045  97.67191
#> [22]  98.14597  98.57784  98.88723  99.14165  99.37138  99.58083  99.76840
#> [29]  99.90769 100.00000
```

#' Plot the autocovariance operator of functional spatial data
#'
#' This function is used to plot the autocovariance operator of functional
#' spatial data along a two-dimensional plane of lags.
#' @param X the functional spatial data. Either an fd object or an array.
#' @param basisobj the basis of the functional data.
#' @param q the maximal lag to be plotted.
#' @param gridsize the resolution of the grid on which the kernel will be
#'   evaluated.
#' @param set.dims the coordinates to be fixed. A vector of dimension r. NaN
#'   represents not fixed components.
#' @param plot.cor a boolean indicating whether the correlation or the
#'   covariance should be plotted.
#' @param na.rm whether or not missing values should be ignored.
#' @param main title of the plot.
#' @param xlab label for the x-axis.
#' @param ylab label for the y-axis.
#' @param ... other arguments to pass to the plot function
#' @seealso \code{\link{fsd.plot.data}}
#' @keywords fsd
#' @import fda
#' @importFrom graphics filled.contour abline axis segments
#' @export
#' @return
#' No return value. The function computes the covariance or correlation
#' operator of the functional spatial data
#' @examples
#' data("temp")
#' fsd.plot.covariance(temp, q = 3, plot.cor = TRUE, zlim = c(-1,1))

fsd.plot.covariance = function (X, basisobj = NULL, q = 0, gridsize = NULL,
                                set.dims = NULL, plot.cor = FALSE,
                                na.rm = FALSE, main = NULL,
                                xlab = NULL, ylab = NULL, ...)
{
  fdnames = NULL
  if (inherits(X, "fd")) {
    basisobj = X$basis
    fdnames = X$fdnames
    X = X$coefs
  }

  if (is.null(basisobj)) {
    basisobj = create.bspline.basis(nbasis = dim(X)[1], norder = 1)
  }

  r = length(dim(X)) - 1

  if (is.null(set.dims)) {
    if (r == 1)
      set.dims = c(NaN)
    else
      set.dims = c(NaN, NaN)
    if (r > 2) {
      set.dims = c(set.dims, rep(0, r - 2))
    }
  }
  if (length(set.dims) != r)
    stop("Need right dimensions")

  if (sum(is.nan(set.dims)) > 2)
    stop("You can only plot two dimensions")

  freedims = which(is.nan(set.dims))
  if (length(freedims) == 0)
    stop("Please provide a dimension to plot")
  only.1d = (length(freedims) == 1)

  q = floor(q)
  if (length(q) == 1) {
    q = rep(q, 2)
  } else if (length(q) > 2) {
    q = q[1:2]
  }

  q[1] = min(q[1], dim(X)[1+freedims[1]]-1)
  if (!only.1d)
    q[2] = min(q[2], dim(X)[1+freedims[2]]-1)
  else
    q[2] = 0

  q0 = rep(0, r)
  q0[freedims] = q[1:length(freedims)]

  inds = lapply(q0, function(w) {-w:w})
  inds[-freedims] = set.dims[!is.nan(set.dims)]
  hlist = unfold(inds)

  singlebasis = (basisobj$nbasis == 1)

  if (singlebasis)
    gridsize = 2

  if (is.null(gridsize))
    gridsize = max(40, ceiling(c(200, 8 * basisobj$nbasis)/(2*min(q) + 1)))

  X = X - apply(X, 1, mean, na.rm = na.rm)

  Clist = lapply(hlist, FUN = fsd.covariance, Y = X,
                 centered = TRUE, na.rm = na.rm)

  gr = (1:gridsize - 0.5) / gridsize
  egr = gr * diff(basisobj$rangeval) + basisobj$rangeval[1]
  lgr = length(gr)

  Cev = lapply(Clist,
               function(C) {eval.bifd(egr, egr,
                                      bifd(coef = C, sbasisobj = basisobj,
                                           tbasisobj = basisobj))})

  if (plot.cor) {
    zeroindex = 1
    if (length(hlist) > 1)
      zeroindex = which(sapply(hlist, function(h) {all(h == 0)}))
    if (length(zeroindex) == 0) {
      C0 = fsd.covariance(Y = X, h = rep(0, r), centered = TRUE)
      vars = diag( eval.bifd(egr, egr, bifd(coef = C0, sbasisobj = basisobj,
                                            tbasisobj = basisobj)) )
    } else {
      vars = diag(Cev[[zeroindex]])
    }
    nm = 1/sqrt(vars%*%t(vars))
    Cev = lapply(Cev, "*", nm)
  }
  bigC = array(dim = lgr* (2*q+1))

  for (k in 1:length(Clist)) {
    bigC[(q[1] + hlist[[k]][freedims[1]]) * lgr + 1:lgr,
         ifelse(!only.1d, q[2] + hlist[[k]][freedims[2]],
                0) * lgr + 1:lgr] = Cev[[k]]
  }
  if (plot.cor)
    bigC = pmin(pmax(bigC, -1), 1)


  if (is.null(main)) {
    if (!plot.cor)
      main = "Covariance Operator Plot"
    else
      main = "Correlation Operator Plot"

    if (any(!is.nan(set.dims))) {
      main = c(main,
               paste("with lags (",
                     paste(gsub(NaN, "*", set.dims), collapse = " , "), ")"))
    }
  }

  if (is.null(xlab)) {
    if (!is.null(fdnames)) {
      xlab = paste("Lag for", names(fdnames)[1+freedims[1]])
    } else {
      xlab = paste("Lag in dimension", freedims[1])
    }
  }

  if (is.null(ylab)) {
    ylab = ""
    if (!only.1d) {
      if (!is.null(fdnames)) {
        ylab = paste("Lag for", names(fdnames)[1+freedims[2]])
      } else {
        ylab = paste("Lag in dimension", freedims[2])
      }
    }
  }

  filled.contour(rep(-q[1]:q[1], each=lgr) + gr, rep(-q[2]:q[2], each=lgr) + gr,
                 bigC, xlim = c(-q[1],q[1]+1), ylim = c(-q[2],q[2]+1),
                 asp = 1, xlab = xlab, ylab = ylab,
                 plot.axes = {d1p = -q[1]:q[1]
                 if (length(d1p) > 10)
                   d1p = pretty(d1p, n = 10)
                 axis(1, at = d1p+0.5, labels = d1p, tick = singlebasis)
                 if (!only.1d) {
                   d2p = -q[2]:q[2]
                   if (length(d2p) > 10)
                     d2p = pretty(d2p, n = 10)
                   axis(2, at = d2p+0.5, labels = d2p, tick = singlebasis)
                 }
                 if (!singlebasis) {
                   segments(-q[1]:(q[1]+1), -q[2], -q[1]:(q[1]+1), q[2]+1,
                            col = 8)
                   segments(-q[1], -q[2]:(q[2]+1), q[1]+1, -q[2]:(q[2]+1),
                            col = 8)
                 }
                 },
                 frame.plot = FALSE, main = main, ...)
}

#' a list containing tuning parameter for the estimation of the the
#' spectral density of indian variable
#'
#' a list containing tuning parameter for the estimation of the the
#' spectral density of indian variable
#'
#'
#' @docType data
#'
#' @usage data(qlist)
#'
#'@format qlist has a dimension of 2 22
#'
#' @format An object of class \code{"matrix" "array"}.
#'
#'
#' @examples
#' data(qlist)
"qlist"

#' Plot functional spatial data
#'
#' This function is used to plot functional spatial data on a grid of dimension
#' r,  along a two-dimensional plane of indices.
#' @param X the functional spatial data. Either an fd object or an array, or a
#'   list thereof.
#' @param basisobj the basis of the functional data.
#' @param gridsize the number of grid points on which to evaluate the functional
#'   data.
#' @param set.dims the coordinates to be fixed. A vector of dimension r. NaN
#'   represents not fixed components.
#' @param xlim limits for the coordinates on the x-axis.
#' @param ylim limits for the coordinates on the y-axis.
#' @param main title of the plot.
#' @param xlab label for the x-axis.
#' @param ylab label for the y-axis.
#' @seealso \code{\link{plot.fsd.fd}}, \code{\link{fsd.plot.filters}},
#'   \code{\link{fsd.plot.covariance}}
#' @keywords fsd
#' @import stats
#' @export
#' @return
#' No return value. This function produces a graphical representation
#' of functional spatial data.
#' @examples
#' data("temp")
#' fsd.plot.data(temp, set.dims = c(NaN, NaN, 3))

fsd.plot.data = function (X, basisobj = NULL, gridsize = NULL,
                          set.dims = NULL, xlim = NULL, ylim = NULL,
                          main = NULL, xlab = NULL, ylab = NULL)
{
  if (!is.list(X) || inherits(X, "fd"))
    X = list(X)

  fdnames = NULL
  basislist = lapply(X, function(Z) {if (inherits(Z, "fd")) Z$basis else NULL})
  namelist = lapply(X, function(Z) {if (inherits(Z, "fd")) Z$fdnames else NULL})
  X = lapply(X, function(Z) {if (inherits(Z, "fd")) Z$coefs else Z})

  if (length(X) > 1) {
    if (sd(sapply(lapply(X, dim), length)) > 0)
      stop("Inconsistent dimensions: all X need to have the same dimensions")
    else if (any(apply(sapply(X, dim), 1, sd) > 0))
      stop("Inconsistent dimensions: all X need to have the same dimensions")
  }

  isfsdobj = !sapply(basislist,is.null)
  if (any(isfsdobj)) {
    basisobj = basislist[[which(isfsdobj)[1]]]
    fdnames = namelist[[which(isfsdobj)[1]]]
  }
  for (k in which(isfsdobj)) {
    if (!(basisobj == basislist[[k]]))
      stop("Inconsistent bases: all X need to have the same basis (or none)")
    if (any(names(fdnames) != names(namelist[[k]])))
      warning("Inconsistent dimension naming: ",
              "not all X have the same dimension names")
  }

  if (is.null(basisobj))
    basisobj = create.bspline.basis(nbasis = dim(X[[1]])[1], norder = 1)

  r = length(dim(X[[1]])) - 1

  if (is.null(set.dims)) {
    if (r == 1)
      set.dims = c(NaN)
    else
      set.dims = c(NaN, NaN)
    if (r > 2) {
      set.dims = c(set.dims, rep(1, r - 2))
    }
  }
  if (length(set.dims) != r)
    stop("The parameter set.dims needs the same length as the dimensions of X")

  if (sum(is.nan(set.dims)) > 2)
    stop("You can only plot two dimensions")

  freedims = which(is.nan(set.dims))
  if (length(freedims) == 0)
    stop("Please provide a dimension to plot")
  only.1d = (length(freedims) == 1)

  if (is.null(xlim))
    xlim = c(1, dim(X[[1]])[1+freedims[1]])
  if (is.null(ylim)) {
    if (!only.1d)
      ylim = c(1, dim(X[[1]])[1+freedims[2]])
    else
      ylim = c(1, 1)
  }

  xlim = floor(xlim)
  ylim = floor(ylim)

  xlim[1] = max(xlim[1], 1)
  xlim[2] = min(xlim[2], dim(X[[1]])[1+freedims[1]])

  if (!only.1d) {
    ylim[1] = max(ylim[1], 1)
    ylim[2] = min(ylim[2], dim(X[[1]])[1+freedims[2]])
  } else {
    ylim = c(1, 1)
  }

  inds = as.list(set.dims)
  inds[[freedims[1]]] = xlim[1]:xlim[2]
  if (!only.1d)
    inds[[freedims[2]]] = ylim[1]:ylim[2]

  Xplot = lapply(X, function(Z) {Y = do.call(what = apply,
                                             args = c(list(X = Z, MARGIN = 1,
                                                           FUN = '['),
                                                      inds))
  if (!is.vector(Y))
    Y = t(Y)
  return(Y)})

  if (only.1d)
    positions = as.list(xlim[1]:xlim[2])
  else
    positions = unfold(list(xlim[1]:xlim[2], ylim[1]:ylim[2]))

  if (is.null(main)) {
    main = "Data Plot"

    if (any(!is.nan(set.dims))) {
      main = c(main,
               paste("with indices (",
                     paste(gsub(NaN, "*", set.dims), collapse = " , "), ")"))
    }
  }

  if (is.null(xlab)) {
    if (!is.null(fdnames)) {
      xlab = names(fdnames)[1+freedims[1]]
    } else {
      xlab = paste("Dimension", freedims[1])
    }
  }

  if (is.null(ylab)) {
    ylab = ""
    if (!only.1d) {
      if (!is.null(fdnames)) {
        ylab = names(fdnames)[1+freedims[2]]
      } else {
        ylab = paste("Dimension", freedims[2])
      }
    }
  }

  if (is.null(gridsize))
    gridsize = max(50, ceiling(c(500, 10 * basisobj$nbasis)/(diff(xlim) + 1)))

  fsd.z.plot(Zlist = Xplot, positions = positions,
             basisobj = basisobj, gridsize = gridsize, main = main,
             xlab = xlab, ylab = ylab)
}

#' Compute the centered version of spatial functional data
#'
#' This function is used to center spatial functional data.
#' @param X The functional spatial data.
#' @param margins The margins for which the centering should be done separately
#'   in every entry, in order to remove a trend.
#' @param na.rm Whether or not missing values should be ignored.
#' @return The centered data.
#' @seealso \code{\link{mean.fsd.fd}}
#' @keywords fsd
#' @export
#' @examples
#' data("temp")
#'
#' T.centered = center.fsd.fd(temp)
#' plot(mean(T.centered, margins = 1:2))
#'
#' T.centered2 = center.fsd.fd(temp, margins = 1:2)
#' plot(mean(T.centered2, margins = 1:2))
#' plot(mean(temp, margins = 1:2))

center.fsd.fd = function (X, margins = NULL, na.rm = FALSE)
{
  if (!inherits(X, "fd"))
    stop("X needs to be functional data")
  if (is.null(margins))
    return(fsd.fd(X$coefs - as.vector(mean.fsd.fd(X, na.rm = na.rm)$coefs),
                  basisobj = X$basis, fdnames = X$fdnames))
  else {
    if (any(margins > length(dim(X$coefs))-1) || any(margins < 1))
      stop("Please provide valid margins")

    r = length(dim(X$coefs)) - 1
    permut = c(1, margins +1, (1:r)[-margins] +1)
    Y = aperm(X$coefs, permut)
    Y.mean = apply(Y, seq(1+length(margins)), mean, na.rm = na.rm)
    Y = aperm(Y-as.vector(Y.mean), order(permut))

    return(fsd.fd(coef = Y,
                  basisobj = X$basis, fdnames = X$fdnames))
  }
}

#' a list containing the maximum lag for the filters for each
#' variable.
#'
#' an example taken from the variable indian which represents a list
#' containing the maximum lag for the filters for each variable.
#' An integer or vector of #' integers.
#'
#'
#' @docType data
#'
#' @usage data(Llist)
#'
#'@format Llist has a dimension of [1:22]
#'
#' @format An object of class \code{int [1:22]}.
#'
#' @examples
#' data(Llist)
"Llist"

#' Check if functional spatial filter is unilateral
#'
#' This function is used to verify if a functional spatial filter is unilateral.
#' @param A the functional spatial filter.
#' @param exclude.origin whether to exclude the origin from the definition of
#'   unilaterality.
#' @return a Boolean indicating whether A is unilateral or not.
#' @seealso \code{\link{fsd.filter}}
#' @keywords fsd
#' @export


fsd.filter.is.unilateral = function (A, exclude.origin = FALSE)
{
  if (!inherits(A, "fsd.filter"))
    stop("Input needs to be a filter object.")

  r = length(A$laglist[[1]])
  lags = matrix(unlist(A$laglist), nrow = r)

  if (exclude.origin)
    if (any(apply(lags == 0, 2, all)))
      return(FALSE)

  for (k in 1:ncol(lags)) {
    for (i in 1:r) {
      if (lags[i,k] < 0)
        return(FALSE)
      else if (lags[i,k] > 0)
        break
    }
  }

  return(TRUE)
}

#' Compute the Fourier transform
#'
#' This function is used to calculate the Fourier transform
#' @param covs the covariances to be transformed
#' @param laglist list of lags.
#' @param thlist list of frequencies.
#' @return the Fourier transform of the covariances.
#' @keywords fsd
#' @export



fsd.fourier = function (covs, laglist, thlist)
{
  if (length(thlist[[1]]) != length(laglist[[1]]))
    stop("Lags and frequencies need to have the same dimension.")
  if (dim(covs)[3] != length(laglist))
    stop("There need to be as many lags as matrices of covariances")

  A = fsd_rcpp_fourier(covs,
                       matrix(unlist(laglist),
                              nrow = length(laglist[[1]])),
                       matrix(unlist(thlist),
                              nrow = length(thlist[[1]])))
  dim(A) = c(dim(covs)[1:2], length(thlist))

  A = sapply(1:dim(A)[3], function(i){A[,,i]}, simplify = FALSE)

  Aobj = list(operators = A, thlist = thlist)
  Aobj
}

#' Calculate the Multivariate Spectral Principal Components Filters Spatial Functional Data
#'
#' This function calculates the multivariate SPC filters from a given spectral density operator.
#' @param SM_FSD list of univariate spatial functional principal component analysis.
#' @param SM_vec list of variable eigenvectors.
#' @param J number of functional multivariate spatial principal components.
#' @return the Multivariate SPC filters.
#' @details The univariate eigenvectors and filters are used to compute multivariate spatial function filters.
#' @keywords mfsd
#' @export

mfsd_Filter = function (SM_FSD, SM_vec, J)
{
  if (!inherits(SM_FSD, "SM_FSD")) {
    stop("'SM_FSD' is not a SM_FSD type.")
  }

  if (!inherits(SM_vec, "SM_vec")) {
    stop("'SM_vec' is not a SM_vec type.")
  }


  y <- length(SM_FSD)

  SML_Filter = vector("list", y)
  oldClass(SML_Filter) <- ("fsd.filter")

  for(i in 1:y){
    SML_Filter[[i]] <- SM_FSD[[i]][["filters"]]
  }


  for (i in 1:y) {
    for (j in 1:length(SM_FSD[[i]]$filters$operators)) {
      tryCatch({
        ei = as.matrix(SM_FSD[[i]][["filters"]][["operators"]][[j]][,1:J])
        rslt <- ei %*%  SM_vec[[i]]
        SML_Filter[[i]][["operators"]][[j]] <- rslt
      }, error = function(e) {
        message("An error occurred in mfsd_Filter while processing the element ", i, ": ", conditionMessage(e), "\n")
      })
    } }

  return(SML_Filter)
}

#' Compute the Multivariate Scores for Spatial Functional Data.
#'
#' @param EEM concatenated scores.
#' @param SM_vec list of variable eigenvectors.
#' @param SM_FSD list of univariate spatial functional principal component analysis.
#' @param X list of univariate spatial functional data.
#' @param J number of functional multivariate spatial principal components.
#' @return a list of the multivariate scores.
#' @keywords mfsd
#' @export

mfsd_Scores = function (EEM, SM_vec, SM_FSD, X, J)
{

  if (!inherits(SM_vec, "SM_vec")) {
    stop("'SM_vec' is not a SM_vec type.")
  }

  if (!inherits(SM_FSD, "SM_FSD")) {
    stop("'SM_FSD' is not a SM_FSD type.")
  }

  y <- length(SM_FSD)

  SML_Scores = vector("list", y)
  oldClass(SML_Scores) <- ("SML_Scores")

  y <- length(SML_Scores)
  var = 1
  for(i in 1:y){
    tryCatch({
      SML_Scores[[i]] <- EEM[,var:(J+var-1)]
      SML_Scores[[i]] <- SML_Scores[[i]] %*% SM_vec[[i]]

      SML_Scores[[i]] <- array(t(SML_Scores[[i]]), dim = c(dim(SML_Scores[[i]])[2],    dim(X[[1]][["coefs"]])[2], dim(X[[1]][["coefs"]])[3]))
      var = var + dim(SM_FSD[[i]][["scores"]])[1]
    }, error = function(e) {
      message("An error occurred in mfsd_Scores while processing the element ", i, ": ", conditionMessage(e), "\n")
    })}

  return(SML_Scores)

}

#' Perform Spectral Principal Components Analysis on Spatial Functional Data
#'
#' This function performs spectral PCA on functional spatial data on a grid of
#' dimension r.
#' @param X the functional spatial data. Either an fd object or an array.
#' @param freq.res the resolution for the computation of the spectral density.
#' @param Npc the number of principal components to be computed.
#' @param L the maximum lag for the filters. An integer or vector of integers.
#' @param q a tuning parameter for the estimation of the the spectral density
#'   operator. An integer or vector of integers.
#' @param na.ignore whether to ignore missing data points in the computation of
#'   the scores.
#' @param return.F a boolean indicating whether to return the spectral density.
#' @param only.filters a boolean indicating whether to compute only the filters,
#'   leaving out the scores.
#' @return A list with components \item{F}{the spectral density.}
#'   \item{tuning.params}{a list of the used tuning parameters \code{freq.res},
#'   \code{q} and \code{Lmax}.} \item{filters}{the SPC filters.}
#'   \item{scores}{the SPC scores.} \item{var}{the theoretical fractions of
#'   variance explained by each PC.} \item{X.mean}{the mean of X.}
#' @seealso \code{\link{fsd.spca.inverse}}, \code{\link{fsd.spectral.density}}
#' @details This function can be used to compute spectral PCA. By setting
#'   \code{q = 0}, it can also be used to perform static PCA.
#'
#'   Setting a higher value for \code{freq.res} increases the runtime
#'   significantly, but yields a more accurate result because of reduced
#'   integration errors.
#' @keywords fsd
#' @export


fsd.spca = function (X, freq.res = 100, Npc = 3, L = 3, q = NULL,
                     na.ignore = TRUE, return.F = FALSE, only.filters = FALSE)
{
  if (any(L < 0))
    stop("Please choose a valid parameter L")
  if (Npc <= 0)
    stop("Please choose a valid parameter Npc")
  if (any(freq.res < 0))
    warning("Please choose a valid frequency resolution")

  if (inherits(X, "fd")) {
    basisobj = X$basis
    meanX = mean.fsd.fd(X, na.rm = TRUE)
    X = X$coefs
    X = X - as.vector(meanX$coefs)
  } else {
    basisobj = NULL
    meanX = apply(X, 1, mean, na.rm = TRUE)
    X = X - meanX
  }

  na.rm = FALSE
  if (anyNA(X)) {
    na.rm = TRUE
    if (na.ignore)
      warning("The data contains missing points which will be ignored ",
              "in the computation of the SPC filters and scores.")
    else
      warning("The data contains missing points which will be ignored ",
              "in the computation of the SPC filters but not the scores.")
  }
  na.ignore = na.ignore & na.rm

  L = floor(L)
  if (length(L) == 1)
    L = rep(L, length(dim(X)) - 1)

  if (length(freq.res) == 1)
    freq.res = rep(freq.res, length(dim(X)) - 1)

  if (!is.null(q)) {
    q = pmax(q, 1)
    q = floor(q)
    freq.res = freq.res * (q > 1)
    L = L * (q > 1)
  }

  if (any(freq.res < 3 * L))
    warning("The chosen frequency resolution is very low.\n",
            "Significant errors during the integration may occur.")

  freq = lapply(freq.res,
                function(fr) {if (fr > 0) -(fr - 1):fr / fr * pi else 0})

  dims = dim(X)[1]
  Npc = min(Npc, dims)

  F = fsd.spectral.density(X, freq, q = q, na.rm = na.rm)
  F$basis = basisobj

  filters = fsd.spca.filters(F, Npc, L)

  scores = NULL
  if (!only.filters)
    scores = fsd.spca.scores(X, filters, na.ignore = na.ignore)

  Fvar = filters$var
  filters$var = NULL

  qF = F$estimation.q

  if (!return.F)
    F = NULL

  na.status = ifelse(na.rm, ifelse(na.ignore, "ignored", "removed"), "none")

  list(F = F, tuning.params = list(freq.res = freq.res,
                                   estimation.q = qF, Lmax = L, na = na.status),
       filters = filters, scores = scores, var = Fvar, X.mean = meanX)
}

#' Plot functional spatial filters
#'
#' This function is used to plot functional spatial filters,  along a
#' two-dimensional plane of indices.
#' @param A the filters.
#' @param basisobj the basis of the functional data.
#' @param gridsize the number of grid points on which to evaluate the functional
#'   data.
#' @param Npc number of filters to plot.
#' @param Lmax maximum lag to plot.
#' @param set.dims the coordinates to be fixed. A vector of dimension r. NaN
#'   represents not fixed components.
#' @param main title of the plot.
#' @param xlab label for the x-axis.
#' @param ylab label for the y-axis.
#' @keywords fsd
#' @export
#' @return
#' No return value. This function produces a graphical representation
#' of functional spatial filters.

fsd.plot.filters = function (A, basisobj = NULL, gridsize = NULL,
                             Npc = 3, Lmax = 3, set.dims = NULL,
                             main = NULL, xlab = NULL, ylab = NULL)
{
  if (inherits(A, "fsd.filter")) {
    basisobj = A$basis
  }

  if (is.null(basisobj))
    basisobj = create.bspline.basis(nbasis = dim(A$operators[[1]])[1],
                                    norder = 1)

  if (is.null(set.dims)) {
    if (length(A$laglist[[1]]) == 1)
      set.dims = c(NaN)
    else
      set.dims = c(NaN, NaN)
    if (length(A$laglist[[1]]) > 2) {
      set.dims = c(set.dims, rep(0, length(A$laglist[[1]]) - 2))
    }
  }
  if (length(set.dims) != length(A$laglist[[1]]))
    stop("Need right dimensions")

  if (sum(is.nan(set.dims)) > 2)
    stop("You can only plot two dimensions")

  freedims = which(is.nan(set.dims))
  if (length(freedims) == 0)
    stop("Please provide a dimension to plot")
  only.1d = (length(freedims) == 1)

  Npc = min(Npc, dim(A$operators[[1]])[2])

  lagrange = apply(matrix(unlist(A$laglist), nrow = length(A$laglist),
                          byrow = TRUE), 2, range)

  if (any(set.dims > lagrange[2,], na.rm = TRUE) ||
      any(set.dims < lagrange[1,], na.rm = TRUE))
    stop("Please choose an available lag")

  plotl = sapply(A$laglist,
                 function(w) {(max(abs((is.nan(set.dims)) * w)) <= Lmax) &&
                     all(w[!is.nan(set.dims)] == set.dims[!is.nan(set.dims)])})
  laglist = lapply(A$laglist[plotl], '[', freedims)

  Alist = lapply(seq(Npc), function(m) {sapply(A$operators[plotl],
                                               function(M) {M[,m]})})

  if (is.null(main)) {
    main = "SFPC Filter Plot"

    if (any(!is.nan(set.dims))) {
      main = c(main,
               paste("with lags (",
                     paste(gsub(NaN, "*", set.dims), collapse = " , "), ")"))
    }
  }

  if (is.null(xlab)) {
    xlab = paste("Lags in dimension", freedims[1])
  }

  if (is.null(ylab)) {
    ylab = ""
    if (!only.1d) {
      ylab = paste("Lags in dimension", freedims[2])
    }
  }

  if (is.null(gridsize))
    gridsize = max(50, ceiling(c(500, 10 * basisobj$nbasis) /
                                 (diff(range(sapply(laglist, '[', 1))) + 1)))

  fsd.z.plot(Zlist = Alist, positions = laglist,
             basisobj = basisobj, gridsize = gridsize, main = main,
             xlab = xlab, ylab = ylab)
}

#' Compute the Scores for Spatial Functional Data
#'
#' This function computes the scores. It applies the matrices of the filter
#' \code{A} to the data \code{X}.
#' @param X the functional spatial data. Either an fd object or an array.
#' @param A the filter used to compute the scores.
#' @param na.ignore whether missing values in the data X should be ignored.
#' @keywords fsd
#' @return the array of the scores.
#' @seealso \code{\link{fsd.spca.inverse}}
#' @export

fsd.spca.scores = function (X, A, na.ignore = FALSE)
{
  if (inherits(X, "fd"))
    X = X$coefs

  r = length(dim(X)) - 1
  d = dim(X)[1]

  if (dim(X)[1] != dim(A$operators[[1]])[1])
    stop("Dimensions are not compatible: Number of basis functions varies")
  if (r != length(A$laglist[[1]]))
    stop("Dimensions are not compatible: Filter lags and data do not fit")

  if (!is.null(A$basis)) {
    B = inprod(A$basis, A$basis)
    X = apply(X, 1+1:r, `%*%`, B)
  }

  Y = array(0, dim = c(dim(A$operators[[1]])[2], dim(X)[1+1:r]),
            dimnames = c(list(paste("PC", 1:dim(A$operators[[1]])[2])),
                         dimnames(X)[1+1:r]))

  if (na.ignore) {
    Xna = apply(is.na(X), 1+1:r, any)
    Y[1:dim(Y)[1] + dim(Y)[1] * rep(which(Xna) - 1, each = dim(Y)[1])] = NA
    X[1:dim(X)[1] + dim(X)[1] * rep(which(Xna) - 1, each = dim(X)[1])] = 0
  }

  for (i in 1:length(A$operators)) {
    if (any(dim(X)[1+1:r] <= abs(A$laglist[[i]])))
      next

    inds1 = as.list(rep(TRUE, 1+r))
    inds2 = inds1

    for (j in 1:length(A$laglist[[i]])) {
      if (A$laglist[[i]][j] > 0) {
        inds1[[1+j]] = -(dim(X)[1+j] + 1 - (1:abs(A$laglist[[i]][j])))
        inds2[[1+j]] = -(1:abs(A$laglist[[i]][j]))
      }
      else if (A$laglist[[i]][j] < 0) {
        inds1[[1+j]] = -(1:abs(A$laglist[[i]][j]))
        inds2[[1+j]] = -(dim(X)[1+j] + 1 - (1:abs(A$laglist[[i]][j])))
      }
    }

    Y = do.call(what = `[<-`,
                args = c(list(Y), inds2,
                         list(do.call(what = `[`,
                                      args = c(list(Y), inds2, drop = FALSE)) +
                                as.vector(t(A$operators[[i]]) %*%
                                            matrix(do.call(what = `[`,
                                                           args = c(list(X), inds1,
                                                                    drop = FALSE)),
                                                   nrow = d)))))
  }

  Y
}

#' Creating a functional spatial filter
#'
#' This function creates a functional spatial filter
#' @param laglist a list of lags.
#' @param operatorlist a list of operators, i.e. matrices.
#' @param basisobj a functional basis object defining the basis.
#' @return the \code{fsd.filter} object.
#' @keywords fsd
#' @export


fsd.filter = function (laglist, operatorlist, basisobj = NULL)
{
  if (length(laglist) != length(operatorlist))
    stop("There need to be as many operator matrices as lags.")

  A = list(laglist = laglist,
           operators = lapply(operatorlist, as.matrix),
           basis = basisobj)
  class(A) = "fsd.filter"
  A
}

#' Compute the mean of spatial functional data
#'
#' This function is used to calculate the mean on functional data on a spatial
#' grid.
#' @param x the functional spatial data.
#' @param margins the margins that should not be averaged over.
#' @param trim currently unused
#' @param na.rm whether or not missing values should be ignored.
#' @param ... currently unused
#' @return the mean of x
#' @seealso \code{\link{center.fsd.fd}}
#' @keywords fsd
#' @export
#' @examples
#' data("temp")
#' plot(mean(temp))
#' plot(mean(temp, margins = 1:2))

mean.fsd.fd = function (x, trim = 0, na.rm = FALSE, ..., margins = NULL)
{
  if (!inherits(x, "fd"))
    stop("x needs to be functional data")
  if (is.null(margins))
    fsd.fd(coef = matrix(apply(x$coefs, 1, mean, na.rm = na.rm), ncol = 1),
           basisobj = x$basis, fdnames = list(x$fdnames[[1]], "mean"))
  else {
    if (any(margins > length(dim(x$coefs))-1) || any(margins < 1))
      stop("Please provide valid margins")
    fsd.fd(coef = apply(x$coefs, c(1, 1 + margins), mean, na.rm = na.rm),
           basisobj = x$basis, fdnames = x$fdnames[c(1, 1 + margins)])
  }
}

#' Calculate the Variance explained by Spectral Principal Components
#'
#' This function estimates the (theoretical) fraction of the variance explained
#' by each spectral principal component for some spatial functional data.
#' @param FS the spectral density.
#' @return the fraction of the variance explained by each PC.
#' @seealso \code{\link{fsd.spca}}
#' @details To ensure accuracy of numerical integration, the frequencies of
#'   \code{FS} should be a suitably dense grid. In order for the estimations to
#'   conform with the actual variance explained (1 - NMSE), the tuning
#'   parameters in the estimation of the spectral density need to be chosen
#'   carefully.
#' @keywords fsd
#' @export

fsd.spca.var = function (FS)
{
  if (!inherits(FS, "fsd.freqdom"))
    stop("FS needs to be a spectral density operator")

  if (!is.null(FS$basis)) {
    B = inprod(FS$basis, FS$basis)
  } else {
    B = diag(dim(FS$operators[[1]])[1])
  }
  B.root = eigen(B)$vectors %*% diag(sqrt(eigen(B)$values)) %*%
    t(eigen(B)$vectors)
  for (i in 1:length(FS$operators)) {
    FS$operators[[i]] = B.root %*% FS$operators[[i]] %*% B.root
  }

  FS.E = sapply(FS$operators,
               function(w) {eigen(w, symmetric = TRUE,
                                  only.values = TRUE)$values},
               simplify="array")

  E.int = apply(pmax(Re(FS.E), 0), 1, sum)
  names(E.int) = paste("PC", 1:dim(FS.E)[1])

  E.int/sum(E.int)
}

#' Plot functional spatial data
#'
#' This function is used to plot functional spatial data on a grid of dimension
#' r,  along a two-dimensional plane of indices.
#' @param x the functional spatial data.
#' @param gridsize the number of grid points on which to evaluate the functional
#'   data.
#' @param set.dims the coordinates to be fixed. A vector of dimension r. NaN
#'   represents not fixed components.
#' @param ... graphical parameters.
#' @seealso \code{\link{fsd.plot.data}}
#' @keywords fsd
#' @export
#'
#' @return
#' No return value. This function is called for its side effect of
#' producing graphical displays of functional spatial data.
#'
#' @examples
#' data("temp")
#' plot(temp)

plot.fsd.fd = function (x, ..., gridsize = NULL, set.dims = NULL)
{
  arglist <- list(...)

  fsd.plot.data(x, gridsize = gridsize, set.dims = set.dims,
                xlim = arglist$xlim, ylim = arglist$ylim, main = arglist$main,
                xlab = arglist$xlab, ylab = arglist$ylab)
}

#' Spatial functional data of sea surface temperature (SST) data
#' for the year 2001, derived from NOAA optimal sea surface
#' temperature interpolation data.
#'
#' Spatial Functional Data of sea surface temperature (SST) data from the NOAA Optimum
#' Interpolation Sea Surface Temperature dataset.
#' The dataset exhibiting typical spatial dependence characterized
#' by an exponential decay.
#'
#'
#' @docType data
#'
#' @usage data(X2001)
#'
#' @format An object of class \code{"fsd.fd"}.
#'
#' @keywords datasets
#'
#'
#' @examples
#' data(X2001)
"X2001"

#' Analyse functional spatial filters
#'
#' This function is used to analyse functional spatial filters, indicating how
#' the squared norms lie.
#' @param object the filter.
#' @param use.norm which norm to use for the lags.
#' @param make.plot a boolean indicating whether to plot the result.
#' @return object list with components \item{balls}{a data frame containing the
#'   cumulative sums of the squared norms of the filter coefficients up to some
#'   maximum lag.} \item{zeroplanes}{a matrix containing the sums of the squared
#'   norms of the filter coefficients lying on the (hyper-) planes with one
#'   dimension of the lag equals zero, along with the sum over all lags as
#'   comparison.} \item{abs.sum}{a data frame containing the cumulative sums of
#'   the norms of the filter coefficients.}
#' @seealso \code{\link{fsd.plot.filters}}
#' @keywords mfsd
#' @export
#' @importFrom graphics par

summary.fsd.filter = function (object, use.norm = Inf, make.plot = FALSE)
{
  npc = dim(object$operators[[1]])[2]
  if (!is.null(object$basis)) {
    B = inprod(object$basis, object$basis)
  } else {
    B = diag(dim(object$operators[[1]])[1])
  }

  if (use.norm < Inf)
    lagnorm = sapply(object$laglist,
                     function(h) {(sum(abs(h)^use.norm))^(1/use.norm)})
  else
    lagnorm = sapply(object$laglist, function(h) {max(abs(h))})

  phinorm = sapply(object$operators, function(M) {diag(t(M) %*% B %*% M)})
  if (npc == 1)
    phinorm = t(phinorm)

  uniquelags = sort(unique(lagnorm))
  normsums = array(dim = c(length(uniquelags), npc),
                   dimnames = list(NULL,
                                   paste0("PC", 1:npc)))

  normsuma = normsums

  for (k in 1:length(uniquelags)) {
    normsums[k, ] = apply(phinorm[ , lagnorm <= uniquelags[k], drop = FALSE], 1,
                          sum)
    normsuma[k, ] = apply(sqrt(phinorm[, lagnorm <= uniquelags[k],
                                       drop = FALSE]), 1, sum)
  }

  normsums2 = sapply(1:length(object$laglist[[1]]),
                     function(i) {apply(phinorm[ , sapply(object$laglist,
                                                          '[', i) == 0,
                                                 drop = FALSE], 1, sum)})
  if (is.vector(normsums2))
    normsums2 = t(normsums2)
  normsums2 = cbind(normsums2, apply(phinorm, 1, sum))

  dimnames(normsums2) = list(paste0("PC", 1:npc),
                             c(paste0("Dim", 1:length(object$laglist[[1]]),
                                      "=0"), "Total"))

  if (make.plot) {
    matplot(uniquelags, normsums, pch = 1, ylim = 0:1,
            xlab = "Lag", ylab = "Sum of squared norms")
    matplot(uniquelags, normsuma, pch = 1,
            xlab = "Lag", ylab = "Sum of norms")
  }

  list(balls = data.frame(max.lag = uniquelags, normsums),
       zeroplanes = t(normsums2),
       abs.sum = data.frame(max.lag = uniquelags, normsuma))
}

#' Analyse multivariate functional spatial filters
#'
#' This function is used to analyse multivariate functional spatial filters, indicating how
#' the squared norms lie.
#' @param object containing a list of variable filters.
#' @param use.norm which norm to use for the lags.
#' @param make.plot a boolean indicating whether to plot the result.
#' @return object list with components \item{balls}{a data frame containing the
#'   cumulative sums of the squared norms of the filter coefficients up to some
#'   maximum lag.} \item{zeroplanes}{a matrix containing the sums of the squared
#'   norms of the filter coefficients lying on the (hyper-) planes with one
#'   dimension of the lag equals zero, along with the sum over all lags as
#'   comparison.} \item{abs.sum}{a data frame containing the cumulative sums of
#'   the norms of the filter coefficients.}
#' @keywords mfsd
#' @export

summary.mfsd.filter = function (object, use.norm = Inf, make.plot = FALSE)
{
  if (!inherits(object, "SM_object")) {
    stop("'object' is not a SM_object type.")
  }

  y <- length(object$filter)
  summary_mfsd = vector("list", y)
  oldClass(summary_mfsd) <- ("summary_mfsd")

  for (i in 1:y) {

    summary_mfsd[[i]] <-  summary.fsd.filter(object[["filter"]][[i]], use.norm = use.norm , make.plot = make.plot)
  }
  summary_mfsd
}

#' Unfold indices into a high-dimensional grid
#'
#' This function is used to expand indices into a grid. Works similar to
#' \code{\link[base]{expand.grid}}.
#' @param indexlist indices to be expanded. Either a list with each member
#'   representing a dimension, or a vector.
#' @param r the dimension of the grid to be created. This is only needed if
#'   indexlist is a vector.
#' @param return.array a boolean indicating whether an array should be returned
#'   or a list.
#' @return a list or an array with the coordinates of the grid.
#' @keywords mfsd
#' @export
#' @examples
#' unfold(list(11:12, 21:23), return.array = TRUE)

unfold = function (indexlist, r = NULL, return.array = FALSE)
{
  if (!is.list(indexlist)) {
    tmpl = list()
    for (i in 1:r) {
      tmpl[[i]] = indexlist
    }
    indexlist = tmpl
  }

  r = length(indexlist)

  gridlist = as.list(indexlist[[1]])

  if (r > 1) {
    for (i in 2:r) {
      gridlist = unlist(lapply(indexlist[[i]],
                               function(x) {lapply(gridlist,
                                                   function(y) {c(y, x)})}),
                        recursive=FALSE)
    }
  }

  if (return.array) {
    return( matrix(unlist(gridlist), nrow = length(gridlist), byrow = TRUE) )
  } else {
    return(gridlist)
  }
}

#' Permute the dimensions of a Functional Spatial Data Grid
#'
#' This function is used to permute the order of the dimensions of the spatial
#' grid of some functional spatial data.
#' @param fsdobj the functional spatial data.
#' @param perm the subscript permutation vector, i.e. a permutation of the
#'   integers 1:r.
#' @seealso \code{\link{fsd.fd}}
#' @keywords fsd
#' @export
#' @return
#' A numeric vector containing the permutation indices used to reorder
#' the functional spatial data.
#' @examples
#' data("temp")
#' fsd.plot.data(fsd.perm(temp, c(2, 1, 3)))

fsd.perm = function (fsdobj, perm = NULL)
{
  if (!inherits(fsdobj, "fsd.fd"))
    stop("fsdobj needs to be an fsd object")

  if (length(dim(fsdobj$coefs)) == 3)
    perm = 2:1
  else if (length(dim(fsdobj$coefs)) <= 2)
    return(fsdobj)

  if (!is.numeric(perm) || length(perm) != length(dim(fsdobj$coefs)) - 1)
    stop("Invalid permutation")

  perm = c(1, 1 + perm)

  fsdobj$coefs = aperm(fsdobj$coefs, perm)
  fsdobj$fdnames = fsdobj$fdnames[perm]

  fsdobj
}

#' Perform Multivariate Spectral Principal Components Analysis on Spatial Functional Data.
#'
#' This function performs multivariate PCA on functional spatial data on a grid of
#' dimension r.
#' @param X a list containing the functional spatial data. Either an fd object or an array.
#' @param freq.res the resolution for the computation of the spectral density.
#' @param SM_Npc the number of multivariat spatial functional principal components to be computed.
#' @param L a list containing the maximum lag for the filters for each variable. An integer or vector of integers.
#' @param q a list containing tuning parameter for the estimation of the the spectral density
#'   operator for each variable. An integer or vector of integers.
#' @param na.ignore whether to ignore missing data points in the computation of
#'   the scores.
#' @param return.F a boolean indicating whether to return the spectral density.
#' @param only.filters a boolean indicating whether to compute only the filters,
#'   leaving out the scores.
#' @return A list with components \item{Value}{eigenvalue for each multivariate PC}
#'   \item{tuning.params}{a list of the used tuning parameters \code{freq.res},
#'   \code{q} and \code{Lmax}.} \item{filters}{the multivariate SPC filters.}
#'   \item{scores}{the multivariate SPC scores.} \item{vm}{the theoretical fractions of
#'   variance explained by each multivariate PC.} \item{ SML_Mean }{the mean of X.}
#'   \item{ SM_FSD }{list of univariate spatial functional principal component analysis.}
#' @details This function can be used to compute spectral PCA. By setting
#'   \code{q = 0}, it can also be used to perform static MFPCA.
#'
#'   Setting a higher value for \code{freq.res} increases the runtime   significantly, but yields a more
#'   accurate result because of reduced integration errors.
#' @keywords mfsd
#' @export
#' @examples
#' \donttest{
#' data("X1999")
#' data("X2000")
#' data("qlist")
#' data("Llist")
#' X = list(X1999, X2000)
#' q = list(qlist[,17], qlist[,18])
#' L = list(L1 = Llist[17], L2 = Llist[18])
#' mfsd(X = X, L = L, q = q, SM_Npc = 4)
#' }
mfsd = function (X, freq.res = 100, SM_Npc = 3, L = 3, q = NULL,
                 na.ignore = TRUE, return.F = FALSE, only.filters = FALSE)
{
  if (!inherits(X, "list")) {
    stop("'X' is not a list.")
  }
  if (!inherits(L, "list")) {
    stop("'L' is not a list.")
  }

  if (length(X)<2) {
    stop("in 'X', the number of variables must be greater or equal than 2.")
  }

  if (!is.numeric(SM_Npc)) {
    stop("Parameter SM_Npc must be integer")
  }


  y <- length(X)
  J = SM_Npc
  var = 1
  SML_Scores = vector("list", y)
  oldClass(SML_Scores) <- ("SML_Scores")

  SML_Filter = vector("list", y)
  oldClass(SML_Filter) <- ("SML_Filter")

  SM_FSD = vector("list", y)
  oldClass(SM_FSD) <- ("SM_FSD")

  SM_vec = vector("list", y)
  oldClass(SM_vec) <- ("SM_vec")

  SML_Mean = vector("list", y)
  oldClass(SML_Mean) <- ("SML_Mean")


  min = X[[1]][["basis"]][["nbasis"]]

  for(i in 1:y){
    if(X[[i]][["basis"]][["nbasis"]] < min){
      min = X[[i]][["basis"]][["nbasis"]]
    } }

  if( SM_Npc > min){
    stop("The parameter SM_Npc must be less than or equal to the number of basis functions")
  }


  if(length(q) > 1){
    for(i in 1:y){
      tryCatch({
        spca = fsd.spca(X[[i]], freq.res = freq.res, q = q[[i]], L = L[[i]], Npc = X[[i]][["basis"]][["nbasis"]], na.ignore = TRUE, return.F = TRUE, only.filters = FALSE)
        SML_Scores[[i]] <- spca$scores
        SML_Filter[[i]] <- spca$filters
        SML_Mean[[i]] <- spca$X.mean


        SM_FSD[[i]] <- list(F = spca$F, tuning.params = spca$tuning.params, filters = spca$filters, scores = spca$scores, var = spca$var, X.mean = spca$X.mean)
      }, error = function(e) {
        message("An error occurred in mfsd while processing the element ", i, ": ", conditionMessage(e), "\n")
      })
    }


  } else{
    for(i in 1:y){
      tryCatch({
        Xspec = fsd.spca(X[[i]], freq.res = freq.res, q = 0, Npc = X[[i]][["basis"]][["nbasis"]], na.ignore = TRUE, return.F = TRUE, only.filters = FALSE)
        SML_Scores[[i]] <- Xspec$scores
        SML_Filter[[i]] <- Xspec$filters
        SML_Mean[[i]] <- Xspec$X.mean


        SM_FSD[[i]] <- list(F = Xspec$F, tuning.params = Xspec$tuning.params, filters = Xspec$filters, scores = Xspec$scores, var = Xspec$var, X.mean = Xspec$X.mean)
      }, error = function(e) {
        message("An error occurred in mfsd while processing the element ", i, ": ", conditionMessage(e), "\n")
      })

    } }


  for(i in 1:y){
    tryCatch({
      SML_Scores[[i]] <- matrix(SM_FSD[[i]][["scores"]], ncol = dim(SM_FSD[[i]][["scores"]])[1], byrow = TRUE)
    }, error = function(e) {
      message("An error occurred in mfsd while processing the element ", i, ": ", conditionMessage(e), "\n")
    }) }

  EEM <- do.call("cbind", SML_Scores)


  zhat <- (1/(dim(EEM)[1]-1))*(t(EEM)%*%EEM)

  evz = eigen(zhat)
  cm = evz$vectors
  vm = evz$values

  val=vm[seq_len(J)]

  vm = 100*(cumsum(vm)/sum(vm))


  vec=cm[,seq_len(J)]


  for(i in 1:y){
    tryCatch({
      SM_vec[[i]] <- vec[var:(J+var-1),]
      var = var + dim(SM_FSD[[i]][["scores"]])[1]
    }, error = function(e) {
      message("An error occurred in mfsd while processing the element ", i, ": ", conditionMessage(e), "\n")
    })}

  var_score= EEM%*%vec
  var_score_m <- array(t(var_score), dim = c(dim(var_score)[2], dim(X[[1]][["coefs"]])[2], dim(X[[1]][["coefs"]])[3]))

  SML_Scores <- mfsd_Scores(EEM, SM_vec, SM_FSD, X, J)


  SML_Filter <- mfsd_Filter(SM_FSD, SM_vec, J)

  if(length(q)<2 && q==0){
    M_object <- list(Value = val, filter = SML_Filter, SM_Scores = SML_Scores,SML_Mean = SML_Mean, SM_FSD = SM_FSD, vm = vm)
    oldClass(M_object) <- ("M_object")

    return(M_object)
  }

  SM_object <- list(Value = val, filter = SML_Filter, SM_Scores = SML_Scores, SML_Mean = SML_Mean, SM_FSD = SM_FSD, vm = vm)
  oldClass(SM_object) <- ("SM_object")
  return(SM_object)
}

#' Estimate the spectral density operator
#'
#' This function is used to estimate the spectral density operator of stationary
#' functional spatial data on a spatial frequency grid.
#' @param X the functional spatial data. Either an fd object or an array.
#' @param freq the spatial frequencies with respect to which the spectral
#'   density is to be estimated. A vector or a list of vectors defining the
#'   grid.
#' @param q the size of the window of the kernel used for estimation. An integer
#'   or a vector. For any dimension, the value \code{q = 1} is equivalent to
#'   \code{q = 0} and means that no lags in this direction are taken into
#'   account.
#' @param na.rm whether or not missing values should be ignored.
#' @return the spectral density, i.e. a list consisting of \item{operators}{a
#'   list of the the spectral density operator at different spatial
#'   frequencies.} \item{freq}{a list of the spatial frequencies.}
#'   \item{basis}{the basis object of the functional data.}
#'   \item{estimation.q}{the tuning parameter \code{q} used in the estimation
#'   process.}
#' @details The spectral density is estimated employing a  Bartlett kernel on
#'   the Euclidean norm of the lags. Therefore, the lag \code{q} is the smallest
#'   lag not to be taken into consideration.
#' @keywords fsd
#' @export

fsd.spectral.density = function (X, freq, q = NULL, na.rm = FALSE)
{
  if (inherits(X, "fd")) {
    basisobj = X$basis
    X = X$coefs
  } else
    basisobj = NULL

  d = dim(X)[1]
  r = length(dim(X)) - 1
  n = dim(X)[1+1:r]

  X = X - apply(X, 1, mean, na.rm = na.rm)

  if (is.null(q))
    q = ceiling(n^0.4)
  else if (length(q) == 1)
    q = rep(q, r)

  q = ceiling(q)

  if (length(q) != r)
    stop("Dimensions don't fit")
  if (any(q < 1))
    stop("Choose a valid q")

  hlist = unfold(lapply(q - 1, function(w) {-w:w}))

  w = sapply(hlist, function(u) {max(0, 1 - sqrt(sum((u/q)^2)))})

  hlist = hlist[w > 0]
  w = w[w > 0]

  Clist = sapply(hlist, FUN = fsd.covariance, Y = X,
                 centered = TRUE, na.rm = na.rm, simplify = "array")

  Clist = Clist * rep(w, each = prod(dim(Clist)[1:2]))

  if (!is.list(freq))
    thlist = unfold(freq, r)
  else
    thlist = unfold(freq)

  hlist = lapply(hlist, "-")

  Fth = fsd.fourier(Clist, hlist, thlist)$operators

  Fobj = list(operators = Fth, freq = thlist,
              basis = basisobj, estimation.q = q)
  class(Fobj) = "fsd.freqdom"
  Fobj
}

#' Simulate a spatial functional ARMA process
#'
#' This function is used to simulate a sample from a spatial functional ARMA
#' (SFARMA) process using a normal or t-distributed noise process.
#'
#' Please note that convergence in the fixed point iteration depends strongly on
#' the sample size and the structure of the AR filter. For certain choices of
#' parameters, even more than \code{n} iterations may be needed.
#' @param n the sample size.
#' @param Sigma the covariance matrix of the noise.
#' @param ARfilter the functional spatial filter of the autoregressive part.
#' @param MAfilter the functional spatial filter of the moving-average part.
#' @param burnin the number of grid points to add on each side of the sample to
#'   achieve stationarity.
#' @param basisobj the basis of the functional data.
#' @param noise the noise distribution to use. Either "normal" or an integer for
#'   the degrees of freedom of a multivariate Student's t-distribution.
#' @param do.fixed.point whether to always use a fixed point iteration.
#' @param max.iter the maximum number of iterations in the fixed point
#'   iteration.
#' @param eps the stopping criterion for the fixed point iteration.
#' @return one sample of the SFARMA process.
#' @seealso \code{\link{fsd.fd}}
#' @keywords fsd
#' @import fda mvtnorm
#' @export
#' @examples

fsd.sfarma = function (n, Sigma = NULL, ARfilter = NULL, MAfilter = NULL,
                       burnin = 30, basisobj = NULL, noise = "normal",
                       do.fixed.point = FALSE, max.iter = 50, eps = 1e-5)
{

  if (is.null(Sigma)) {
    if (!is.null(ARfilter))
      d = dim(ARfilter$operators[[1]])[1]
    else if (!is.null(MAfilter))
      d = dim(MAfilter$operators[[1]])[1]
    else
      stop("Not enough parameters")
    Sigma = diag(exp(- (1:d -1) / 10))
  }

  Sigma = as.matrix(Sigma)

  d = dim(Sigma)[1]
  r = length(n)

  if (is.null(basisobj)) {
    basisobj = create.bspline.basis(nbasis = d, norder = 1)
  }

  if (!inherits(basisobj, "basisfd"))
    stop("basisobj needs to be a basis.")

  if (!is.null(ARfilter) && !inherits(ARfilter, "fsd.filter"))
    stop("ARfilter needs to be a filter object.")
  if (!is.null(MAfilter) && !inherits(MAfilter, "fsd.filter"))
    stop("MAfilter needs to be a filter object.")

  n2 = n + 2*burnin

  if (!is.numeric(noise) || noise == 0)
    Z = array(t(rmvnorm(prod(n2), sigma = Sigma)),
              dim = c(d, n2))
  else
    Z = array(t(rmvt(prod(n2), sigma = Sigma, df = noise)),
              dim = c(d, n2))

  if (!is.null(MAfilter))
    Z = fsd.spca.scores(X = Z, A = MAfilter)

  X = Z

  if (!is.null(ARfilter)) {
    if (!fsd.filter.is.unilateral(ARfilter) && !do.fixed.point) {
      message("Multilateral AR-filter necessitates fixed point iterations...")
      do.fixed.point = TRUE
    }
    if (!do.fixed.point) {
      message("Unilateral AR-filter permits ordinary recursive computation.")
      splitlag = split(unlist(ARfilter$laglist),f = seq(r))
      minInd = 1 + sapply(splitlag, max)
      maxInd = n2 + sapply(splitlag, min)
      for (i in 1:prod(n2)) {
        ind = arrayInd(i, n2)
        if (any(ind < minInd) || any(ind > maxInd))
          next

        for (k in 1:length(ARfilter$operators)) {
          if (all(ARfilter$laglist[[k]] == 0))
            next
          X = do.call(what = `[<-`,
                      args = c(list(X), TRUE, ind,
                               list(do.call(what = `[`, args = c(list(X), TRUE,
                                                                 ind)) +
                                      ARfilter$operators[[k]] %*%
                                      do.call(what = `[`,
                                              args = c(list(X), TRUE,
                                                       ind - ARfilter$laglist[[k]])))))
        }
      }
    } else {
      message("Starting fixed point iterations with eps = ", eps,
              " and max.iter = ", max.iter)
      convergent = FALSE
      for (i in 1:max.iter) {
        Xold = X
        X = fsd.spca.scores(X = X, A = ARfilter) + Z
        difference = max(fsd.norm(fsd.fd(X - Xold, basisobj = basisobj)))
        if (difference < eps) {
          convergent = TRUE
          break
        }
      }
      if (convergent)
        message("Convergence reached after ", i, " iterations. Difference: ",
                format(difference, digits = 5))
      else
        message("No convergence after ", i, " iterations. Difference: ",
                format(difference, digits = 5))
    }
  }

  if (length(dim(X)) < r + 1)
    X = array(X, dim = c(1, dim(X)))

  if (burnin > 0) {
    if (r == 1)
      inds = list(seq(n) + burnin)
    else
      inds = lapply(lapply(X = n, FUN = seq), "+", burnin)
    X = do.call(`[`, c(list(X), TRUE, inds, drop = FALSE))
  }

  dimnames(X) = list(basisobj$names)

  fsd.fd(X, basisobj)
}

#' Compute the Fourier inverse
#'
#' This function is used to calculate the Fourier inverse.
#' @param evecs the eigenvectors to be expanded.
#' @param thlist list of frequencies.
#' @param laglist list of lags.
#' @return the filter obtained by applying the Fourier inverse on evecs.
#' @keywords fsd
#' @export


fsd.fourier.inverse = function (evecs, thlist, laglist)
{
  if (length(thlist[[1]]) != length(laglist[[1]]))
    stop("Lags and frequencies need to have the same dimension.")
  if (dim(evecs)[3] != length(thlist))
    stop("There need to be as many frequencies as matrices of eigenvectors.")

  A = Re(fsd_rcpp_fourier_inverse(evecs,
                                  matrix(unlist(thlist),
                                         nrow = length(thlist[[1]])),
                                  matrix(unlist(laglist),
                                         nrow = length(laglist[[1]]))))
  dim(A) = c(dim(evecs)[1:2], length(laglist))

  A = sapply(1:dim(A)[3], function(i){A[,,i]}, simplify = FALSE)

  Aobj = list(operators = A, laglist = laglist)
  class(Aobj) = "fsd.filter"
  Aobj
}

#' Reconstruct the original functional data from the filters and the score
#'
#' This function is used to reconstruct the original functional data from the
#' filters and the score. It applies the transposed matrices of the filter
#' \code{A} to the scores.
#' @param A the filters.
#' @param scores the scores as an array.
#' @param mean.X the mean of X.
#' @return X the reconstructed functional data
#' @seealso \code{\link{fsd.spca}}, \code{\link{fsd.spca.scores}}
#' @keywords fsd
#' @export

fsd.spca.inverse = function (A, scores, mean.X = NULL)
{
  if (!inherits(A, "fsd.filter"))
    warning("A is not of the class fsd.filter")

  if (dim(scores)[1] > dim(A$operators[[1]])[2])
    stop("Dimensions are not compatible: Number of PC too small")
  if (length(dim(scores)) - 1 != length(A$laglist[[1]]))
    stop("Dimensions are not compatible: Filter lags and scores do not fit")

  if (dim(scores)[1] < dim(A$operators[[1]])[2]) {
    message("Less scores than PC filters provided. ",
            "Remaining scores are assumed to be zero.")
    dimdiff = dim(A$operators[[1]])[2] - dim(scores)[1]
    scores = apply(scores, 2:length(dim(scores)),
                   function(y) {c(y, rep(0, dimdiff))})
  }

  A2 = A
  A2$laglist = lapply(A2$laglist, "-")
  A2$operators = lapply(A2$operators, t)
  A2$basis = NULL

  X = fsd.spca.scores(scores, A2)

  if (!is.null(mean.X)) {
    if (inherits(mean.X, "fd"))
      mean.X = mean.X$coefs
    if (length(mean.X) == dim(X)[1]) {
      X = X + as.vector(mean.X)
    } else {
      warning("Mean X does not have the correct dimension")
    }
  }

  dimnames(X)[[1]] = sprintf("f%s", 1:dim(X)[1])

  if (!is.null(A$basis)) {
    dimnames(X)[[1]] = A$basis$names
    X = fsd.fd(X, A$basis)
  }

  X
}

#' Plot functional spatial filters
#'
#' This function plots functional spatial filters along a
#' two-dimensional plane of indices.
#'
#' @param x the filter object.
#' @param xlim limits of the x-axis.
#' @param ylim limits of the y-axis.
#' @param main main title of the graph.
#' @param xlab label of the x-axis.
#' @param ylab label of the y-axis.
#' @param gridsize number of grid points used to evaluate the functional data.
#' @param Npc number of filters to plot.
#' @param Lmax maximum lag to plot.
#' @param set.dims coordinates to be fixed. A vector of dimension r. `NaN`
#' represents components that are not fixed.
#' @param ... additional graphical parameters.
#'
#' @seealso \code{\link{fsd.plot.filters}}
#'
#' @method plot fsd.filter
#' @export
#' @importFrom graphics plot
#'
#' @return
#' No return value. This function is called for its side effect of
#' producing graphical displays of functional spatial filters.

plot.fsd.filter <- function(
    x,
    xlim = NULL,
    ylim = NULL,
    main = NULL,
    xlab = NULL,
    ylab = NULL,
    gridsize = NULL,
    Npc = 3,
    Lmax = 3,
    set.dims = NULL,
    ...
) {

  fsd.plot.filters(
    x,
    gridsize = gridsize,
    Npc = Npc,
    Lmax = Lmax,
    set.dims = set.dims,
    main = main,
    xlab = xlab,
    ylab = ylab
  )

}

#' Creating a functional spatial data object
#'
#' This function extends the possibilities of the function fd found in the
#' package fda to include functional data on a spatial grid of arbitrary
#' dimension r.
#' @param coef an array of arbitrary dimension 1 + r, where the first dimension
#'   corresponds to basis functions.
#' @param basisobj a functional basis object defining the basis.
#' @param fdnames a list of length r with each member containing the labels for
#'   the dimensions of the data.
#' @return the \code{fsd.fd} object.
#' @keywords fsd
#' @import fda
#' @export

fsd.fd = function (coef = NULL, basisobj = NULL, fdnames = NULL)
{
  if (is.null(coef) && is.null(basisobj))
    basisobj <- create.bspline.basis(nbasis = 2, norder = 1)
  if (is.null(coef))
    coef <- rep(0, basisobj[["nbasis"]])
  type <- basisobj$type
  {
    if (!is.numeric(coef))
      stop("'coef' is not numeric.")
    else if (is.vector(coef)) {
      coef <- as.matrix(coef)
      if (identical(type, "constant"))
        coef <- t(coef)
      coefd <- dim(coef)
      ndim <- length(coefd)
    }
    else if (is.array(coef)) {
      coefd <- dim(coef)
      ndim <- length(coefd)
    }
    else stop("Type of 'coef' is not correct")
  }
  if (is.null(basisobj)) {
    dimC <- dim(coef)
    nb <- {
      if (is.null(dimC))
        length(coef)
      else dimC[1]
    }
    basisobj <- create.fourier.basis(nbasis = nb)
    type <- basisobj$type
  }
  else if (!(inherits(basisobj, "basisfd")))
    stop("Argument basis must be of basis class")
  nbasis <- basisobj$nbasis
  dropind <- basisobj$dropind
  ndropind <- length(basisobj$dropind)
  if (coefd[1] != nbasis - ndropind)
    stop("First dim. of 'coef' not equal to 'nbasis - ndropind'.")
  if (ndim > 1)
    nrep <- coefd[2]
  else nrep <- 1
  if (ndim > 2)
    nvar <- coefd[3]
  else nvar <- 1
  if (is.null(fdnames)) {
    fdnames = NULL
    if (!is.vector(coef)) {
      fdnames = dimnames(coef)
    }
    if (is.null(fdnames)) {
      fdnames <- c("time",
                   lapply(1:(ndim-1),
                          function(k) {paste0("Dim", k, " ",
                                              as.character(1:coefd[1+k]))}))
    }
    for (i in 2:length(fdnames) - 1) {
      if (is.null(fdnames[[i+1]]))
        fdnames[[i+1]] = paste0("Dim", i, " ", as.character(1:coefd[1+i]))
    }
    if (is.null(names(fdnames))) {
      names(fdnames) <- c("args", paste0("Dim", as.character(1:(ndim-1))))
    }
  }
  if (is.null(dimnames(coef))) {
    dimc <- dim(coef)
    ndim <- length(dimc)
    dnms <- vector("list", ndim)
    rdms <- mapply(function(a,b) {a == length(b)}, dimc, fdnames)
    dnms[[1]] = basisobj$names
    for (i in which(rdms)) {
      dnms[[i]] <- fdnames[[i]]
    }
    dimnames(coef) <- dnms
  }
  fdobj <- list(coefs = coef, basis = basisobj, fdnames = fdnames)
  oldClass(fdobj) <- c("fsd.fd", "fd")
  fdobj
}


#' Converting a functional data object into a functional spatial data object
#'
#' Works as one would expect
#' @param x a functional data object or something that can be converted into
#'   one.
#' @param ... currently unused
#' @return An object of class \code{fsd.fd}.
#' The returned object represents functional data indexed over a spatial grid.
#' It contains: \code{coefs}: an array of coefficients where the first dimension
#'  corresponds to basis coefficients and the remaining dimensions correspond
#'  to spatial locations,
#' \code{basis}: a basis object describing the functional representation,
#' \code{fdnames}: a list describing the axes (spatial indices and function domain).
#' This transformation embeds classical functional data into a spatial framework.
#' @seealso \code{\link{fsd.fd}}
#' @keywords fsd
#' @export

as.fsd.fd = function (x, ...)
{
  if (!inherits(x, "fd"))
    x = as.fd(x)

  r = length(dim(x$coefs)) - 1

  fdnames = x$fdnames[1:(r+1)]

  fsd.fd(coef = x$coefs, basisobj = x$basis, fdnames = fdnames)
}


#' Converting a functional spatial data object into a functional data object
#'
#' Works as one would expect
#' @param x a functional spatial data object
#' @param ... currently unused
#' @return An object of class \code{fd} (from the \pkg{fda} package).
#' The returned object corresponds to a functional data object obtained by
#' collapsing the spatial dimensions of the input.
#' The coefficients matrix represents functional observations, while the
#' spatial structure is removed (or flattened).
#' @seealso \code{\link{fsd.fd}}
#' @keywords fsd
#' @import fda
#' @export

as.fd.fsd.fd = function (x, ...)
{
  if (!inherits(x, "fsd.fd"))
    stop("x needs to be a functional spatial data object")

  x = x[drop = TRUE]

  r = length(dim(x$coefs)) - 1

  if (r > 2)
    stop("x can only have a two-dimensional spatial grid")

  if (length(x$fdnames) == 2)
    x$fdnames = c(x$fdnames, list(funs = "values"))
  if (length(x$fdnames) > 3)
    x$fdnames = x$fdnames[1:3]

  fd(x$coefs, basisobj = x$basis, fdnames = x$fdnames)
}


#' Subsetting a functional spatial data object
#'
#' Subsetting works as with conventional arrays. By default, dimensions can be
#' dropped.
#' @param fsdobj a functional spatial data object
#' @param ... the indices to take
#' @param drop whether to flatten the spatial grid by dropping dimensions of
#'   length one.
#' @return An object of class \code{fsd.fd}.
#' The returned object is a subset of the original functional spatial data object.
#' The structure of the coefficient array and the associated spatial grid is
#' reduced according to the selected indices. If \code{drop = TRUE}, dimensions
#' of length one in the spatial grid may be removed.
#' @keywords fsd
#' @export
#' @examples
#' data("temp")
#' plot(temp[1:2, 1:2, 1, drop = TRUE])

`[.fsd.fd` = function (fsdobj, ..., drop = TRUE)
{
  args = lapply( eval(substitute(alist(...))) ,
                 function(y) {if (is.call(y))
                   eval.parent(y, n = 3)
                   else {
                     if (length(y) > 1)
                       y
                     else if (nchar(as.character(y)) == 0)
                       y
                     else
                       eval.parent(y, n = 3)
                   }})

  r = length(dim(fsdobj$coefs)) - 1

  inds = c(TRUE, args)
  if (length(inds) == 1)
    inds = c(inds, rep(TRUE, r))

  fsdobj$coefs = do.call(what = '[', args = c(list(fsdobj$coefs),
                                              inds, drop = drop))

  fsdobj$fdnames = fsdobj$fdnames[1:(r+1)]

  fsdobj$fdnames = mapply('[', fsdobj$fdnames, inds, SIMPLIFY = FALSE)

  if (drop) {
    collapsing = (sapply(fsdobj$fdnames, length) == 1)
    collapsing[1] = FALSE
    if (is.vector(fsdobj$coefs)) {
      collapsing[2] = FALSE
      fsdobj$coefs = as.matrix(fsdobj$coefs)
    }
    fsdobj$fdnames = fsdobj$fdnames[!collapsing]
  }

  fsdobj
}


#' Subsetting a functional spatial data object
#'
#' Take only every \code{s}-th element
#' @param fsdobj a functional spatial data object
#' @param s an integer or a vector with one entry for every dimension of the
#'   spatial grid.
#' @return An object of class \code{fsd.fd}.
#' The returned object is a downsampled version of the input functional spatial
#' data object, where only every \code{s}-th spatial location is retained along
#' each spatial dimension.
#' @keywords fsd
#' @export
#' @examples
#' data("temp")
#' plot(temp %/% 3)

`%/%.fsd.fd` = function (fsdobj, s)
{
  if(!inherits(fsdobj, "fsd.fd"))
    stop("Needs functional spatial data")
  if (length(s) > 1 && length(s) != length(dim(fsdobj$coefs)) -1 )
    stop("Incompatible dimensions")
  if (any(s < 1) || any(s %% 1 != 0))
    stop("Needs positive integer s")

  r = length(dim(fsdobj$coefs)) - 1

  if (length(s) == 1)
    s = rep(s, r)

  inds = lapply(1:r, function(i) {seq(from = 1, to = dim(fsdobj$coefs)[1+i],
                                      by = s[i])})

  do.call('[', c(list(fsdobj), inds, drop = FALSE))
}


#' Arithmetic for functional spatial data objects
#'
#' Works as one would expect.
#' @return An object of class \code{fsd.fd}.
#' The result corresponds to the element-wise sum of the coefficient arrays of
#' the two functional spatial data objects. The spatial structure and basis are preserved.
#' @param fsd1,fsd2 functional spatial data objects
#' @keywords fsd
#' @export

`+.fsd.fd` = function (fsd1, fsd2)
{
  if (!inherits(fsd1, "fsd.fd") || !inherits(fsd2, "fsd.fd"))
    stop("Needs compatible data")
  else if (!(fsd1$basis == fsd2$basis))
    stop("Incompatible bases")

  if (length(dim(fsd1$coefs)) != length(dim(fsd2$coefs))) {
    if (length(dim(fsd2$coefs)) == 2 && dim(fsd2$coefs)[2] == 1)
      fsd1$coefs = fsd1$coefs + as.vector(fsd2$coefs)
    else if (length(dim(fsd1$coefs)) == 2 && dim(fsd1$coefs)[2] == 1) {
      tmp = as.vector(fsd1$coefs)
      fsd1 = fsd2
      fsd1$coefs = fsd1$coefs + tmp
    } else
      stop("Incompatible dimensions")
  }
  else if (any(dim(fsd1$coefs) != dim(fsd2$coefs)))
    stop("Incompatible dimensions")
  else
    fsd1$coefs = fsd1$coefs + fsd2$coefs

  fsd1
}


#' Arithmetic for functional spatial data objects
#'
#' Works as one would expect.
#' @return An object of class \code{fsd.fd}.
#' If a single argument is provided, the result is the additive inverse of the
#' input object. Otherwise, it corresponds to the element-wise difference between
#' two functional spatial data objects.
#' @param fsd1,fsd2 functional spatial data objects
#' @keywords fsd
#' @export

`-.fsd.fd` = function (fsd1, fsd2 = NULL)
{
  if (is.null(fsd2)) {
    fsd1$coefs = -fsd1$coefs
    fsd1
  } else {
    fsd2 = -fsd2
    fsd1 + fsd2
  }
}


#' Arithmetic for functional spatial data objects
#'
#' Works as one would expect.
#' @return An object of class \code{fsd.fd}.
#' The result corresponds to a scalar multiplication of the functional spatial
#' data object, where all coefficients are multiplied by the given scalar.
#' @param scalar a scalar number
#' @param fsdobj a functional spatial data object
#' @keywords fsd
#' @export

`*.fsd.fd` = function (scalar, fsdobj)
{
  if (is.numeric(fsdobj)) {
    tmp = fsdobj
    fsdobj = scalar
    scalar = tmp
  }

  if (!is.numeric(scalar) ||
      length(scalar) != 1 ||
      !inherits(fsdobj, "fsd.fd"))
    stop("Only scalar multiplication with an fsd.fd object is possible")

  fsdobj$coefs = scalar * fsdobj$coefs

  fsdobj
}


#' Arithmetic for functional spatial data objects
#'
#' Works as one would expect.
#' @param fsdobj a functional spatial data object
#' @param scalar a scalar number
#' @return An object of class \code{fsd.fd}.
#' The result corresponds to a scalar division of the functional spatial data
#' object, where all coefficients are divided by the given scalar.
#' @keywords fsd
#' @export

`/.fsd.fd` = function (fsdobj, scalar)
{
  if (!is.numeric(scalar) ||
      length(scalar) != 1 ||
      !inherits(fsdobj, "fsd.fd"))
    stop("Only scalar division of an fsd.fd object is possible")

  fsdobj$coefs = fsdobj$coefs / scalar

  fsdobj
}

#' Estimate autocovariance operators
#'
#' This function is used to estimate a covariance operator of stationary
#' functional spatial data with respect to some lag h.
#' @param Y the functional spatial data. Either an fd object or an array.
#' @param h the spatial lag with respect to which the autocovariance is to be
#'   estimated.
#' @param centered a boolean indicating whether or not the data is already
#'   centered.
#' @param na.rm whether or not missing values should be ignored.
#' @param unbiased whether or not to scale the estimator to be unbiased. FALSE
#'   by default. Is not used if na.rm is TRUE.
#' @return the covariance matrix.
#' @keywords fsd
#' @export


fsd.covariance = function (Y = NULL, h, centered = FALSE, na.rm = FALSE,
                           unbiased = FALSE)
{
  if (inherits(Y, "fd"))
    Y = Y$coefs

  r = length(dim(Y)) - 1
  n = dim(Y)[1+1:r]
  d = dim(Y)[1]

  if (any(n - abs(h) <= 0))
    return(NULL)

  if (!centered)
    Y = Y - apply(Y, 1, mean, na.rm = na.rm)

  if (na.rm) {
    Yna = apply(is.na(Y), 1+1:r, any)
    Y[1:dim(Y)[1] + dim(Y)[1] * rep(which(Yna) - 1, each = dim(Y)[1])] = 0
  }

  ind2 = mapply(function(ni, hi){max(-hi, 0) + 1:(ni - abs(hi))}, n, h,
                SIMPLIFY = FALSE)
  ind1 = mapply("+", ind2, h, SIMPLIFY = FALSE)

  if (any(sapply(ind1, length) > 1)) {
    Ch = crossprod(do.call(what = apply, args = c(X = list(Y), MARGIN = 1,
                                                  FUN = '[', ind1)),
                   do.call(what = apply, args = c(X = list(Y), MARGIN = 1,
                                                  FUN = '[', ind2)))
  } else {
    Ch = tcrossprod(do.call(what = apply, args = c(X = list(Y), MARGIN = 1,
                                                   FUN = '[', ind1)),
                    do.call(what = apply, args = c(X = list(Y), MARGIN = 1,
                                                   FUN = '[', ind2)))
  }

  if (na.rm)
    Ch / sum( !(do.call('[', c(list(Yna), ind1)) |
                  do.call('[', c(list(Yna), ind2))) )
  else if (unbiased)
    Ch / prod(n - abs(h))
  else
    Ch / prod(n)
}

#' Temperature Data in Wyoming
#'
#' CPC daily Temperature data of mean temperatures in Wyoming
#' projected onto a B-spline basis of 36 functions for each year,
#' from 1979 to 2017 and with a spatial resolution of 0.5 degrees.
#'
#' The gridded data has been obtained by Shepard's method.
#'
#' @docType data
#'
#' @usage data(temp)
#'
#' @format An object of class \code{"fsd.fd"}.
#'
#' @keywords datasets
#' @examples
#' data(temp)
#' plot(temp)
"temp"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

fsd_rcpp_fourier <- function(A, lags, freq) {
    .Call('_MFSD_fsd_rcpp_fourier', PACKAGE = 'MFSD', A, lags, freq)
}

fsd_rcpp_fourier_inverse <- function(B, freq, lags) {
    .Call('_MFSD_fsd_rcpp_fourier_inverse', PACKAGE = 'MFSD', B, freq, lags)
}

#' Perform A Comparative Evaluation of Dimension Reduction: SMFPCA versus MFPCA in Multivariate Spatial Data Reconstruction
#'
#' To assess the effectiveness of integrating the spatial and multivariate aspects through SMFPCA, it is
#' common to reconstruct the original functional data from the already computed scores and filters.
#' For each instance, we employ both the novel SMFPCA and the conventional MFPCA. We assess the
#' effectiveness of dimension reduction using the metric known as the normalized mean squared
#' error (NMSE) for each univariate component. NMSE* evaluates the quality of the approximation without being
#' influenced by grid boundary effects, unlike NMSE, where the approximation is
#' less accurate at the boundary.
#' @param Xspec an object of class SM_object, this function is obtained by applying the mfsd
#'  function with a non-zero value of q.
#' @param Xspecc an object of class M_object, this object is obtained by applying the mfsd
#'  function with q equal to zero.
#' @param X00 a list containing the functional spatial data. Either an fd object or an array.
#' @param Npc the number of multivariat spatial functional principal components to be computed.
#' @return a display of the NMSE and NMSE* results from \code{\link{print}}.
#' @keywords mfsd
#' @export

mfsd.evaluation = function (Xspec, Xspecc, X00, Npc = 3 )
{


  if (!inherits(Xspec, "M_object")) {
    stop("'Xspec' is not a M_object type.")
  }

  if (!inherits(Xspecc, "SM_object")) {
    stop("'Xspec' is not a SM_object type.")
  }


  if (!is.numeric(Npc)) {
    stop("Parameter Npc must be integer")
  }

  y <- length(Xspec$SM_Scores)
  for (j in 1:y) {
    if (!inherits(X00[[j]], "fsd.fd")) {
      stop("'X00' is not a fsd.fd type.")
    } }

  X = X00
  Npc = Npc


  for (j in 1:y) {

    paste("variable", j)

    X = X00[[j]]

    suppressMessages({

      Xstat.r = lapply(1:Npc,
                       function(k) {fsd.spca.inverse(Xspec[["filter"]][[j]],
                                                     Xspec[["SM_Scores"]][[j]][1:k,,,
                                                                               drop = FALSE],
                                                     Xspec[["SML_Mean"]][[j]])})

      Xspec.r = lapply(1:Npc,
                       function(k) {fsd.spca.inverse(Xspecc[["filter"]][[j]],
                                                     Xspecc[["SM_Scores"]][[j]][1:k,,,
                                                                                drop = FALSE], Xspecc[["SML_Mean"]][[j]])})

    })


    norm.st = sapply(Xstat.r, function(Xr) {fsd.norm(Xr - X)^2},
                     simplify = FALSE)
    norm.sp = sapply(Xspec.r, function(Xr) {fsd.norm(Xr - X)^2},
                     simplify = FALSE)


    mse.stat = sapply(norm.st, mean)
    mse.spec = sapply(norm.sp, mean)
    normX = mean(fsd.norm(X)^2)


    message(rbind("ADVANTAGE" = (mse.stat-mse.spec) / normX,
                "NMSE_spat" = mse.spec / normX,
                "NMSE_ord"  = mse.stat / normX,
                "NMSE_spat*"= 1 - cumsum(Xspecc[["SM_FSD"]][[j]][["var"]][1:Npc]), # aproximation error N2
                "NMSE_ord*" = 1 - cumsum(Xspec[["SM_FSD"]][[j]][["var"]][1:Npc])))


  }
}

#' Plot functional data on a grid
#'
#' This function plots functional data on a grid.
#' @param Zlist list of arrays with coefficients.
#' @param positions the positions of the single curves in space.
#' @param basisobj the basis of the functional data.
#' @param gridsize the number of grid points on which to evaluate the curves.
#' @param ... other arguments to pass to the plot function
#' @keywords fsd
#' @export
#' @import stats fda
#' @importFrom graphics plot lines abline axis
#'
#' @return
#' No return value. This function is called for its side effect of
#' producing a graphical representation of functional data evaluated
#' on a spatial grid.

fsd.z.plot = function (Zlist, positions, basisobj = NULL, gridsize = 50, ...)
{
  if (is.null(basisobj))
    basisobj = create.bspline.basis(nbasis = dim(Zlist[[1]])[1], norder = 1)

  egr = 0:gridsize / gridsize
  egr2 = egr * diff(basisobj$rangeval) + basisobj$rangeval[1]

  Zev = sapply(Zlist, function(x) {eval.fd(egr2, fd(coef = x,
                                                    basisobj = basisobj))},
               simplify = "array")

  if (is.vector(Zev))
    Zev = matrix(Zev, ncol = 1)

  Zev = Zev/max(abs(Zev), na.rm = TRUE) /2 *0.95

  only.1d = (length(positions[[1]]) == 1)

  lags1 = unique(sapply(positions, '[', 1))
  lags2 = 0
  if (!only.1d)
    lags2 = unique(sapply(positions, '[', 2))

  xlim = c( min(lags1) , max(lags1) )
  ylim = c( min(lags2) , max(lags2) )

  plot(NULL, NULL,
       xlim = xlim + c(-0.5, 0.5), ylim = ylim + c(-0.5, 0.5),
       xaxt = 'n', yaxt = 'n', frame.plot = FALSE, ...)

  d1p = xlim[1]:xlim[2]
  if (length(d1p) > 10)
    d1p = pretty(d1p, n = 10)
  axis(1, at = d1p, labels = d1p, tick = FALSE)
  if (!only.1d) {
    d2p = ylim[1]:ylim[2]
    if (length(d2p) > 10)
      d2p = pretty(d2p, n = 10)
    axis(2, at = d2p, labels = d2p, tick = FALSE)
  }

  abline(h = lags2, v = lags1, col = 8, lty = 3)

  for (m in length(Zlist):1) {
    for (i in 1:dim(Zev)[2]) {
      lines(positions[[i]][1] + (egr -0.5)*0.8,
            ifelse(!only.1d, positions[[i]][2], 0) + Zev[ , i, m], col = m)
    }
  }
}

#' Calculate the Covariance of the Spectral Principal Component Scores
#'
#' This function estimates the (theoretical) autocovariance for each spectral
#' principal component score for some spatial functional data.
#' @param FS the spectral density.
#' @param L the maximum lag to compute the covariance of the scores
#' @return A list with components \item{laglist}{the list of lags.} \item{cov}{a
#'   matrix with the autocovariances of the principal component scores.}
#' @seealso \code{\link{fsd.spca}}, \code{\link{fsd.spca.var}}
#' @details To ensure accuracy of numerical integration during the Fourier
#'   transform, the frequencies of \code{F} should be a suitably dense grid.
#' @keywords fsd
#' @export

fsd.spca.cov = function (FS, L = 3)
{
  if (!inherits(FS, "fsd.freqdom"))
    stop("FS needs to be a spectral density operator")

  if (length(L) == 1)
    L = rep(L, length(FS$freq[[1]]))

  if (length(L) != length(FS$freq[[1]]))
    stop("Dimensions incompatible")

  if (!is.null(FS$basis)) {
    B = inprod(FS$basis, FS$basis)
  } else {
    B = diag(dim(FS$operators[[1]])[1])
  }
  B.root = eigen(B)$vectors %*% diag(sqrt(eigen(B)$values)) %*%
    t(eigen(B)$vectors)
  for (i in 1:length(FS$operators)) {
    FS$operators[[i]] = B.root %*% FS$operators[[i]] %*% B.root
  }

  FS.E = sapply(FS$operators,
               function(w) {eigen(w, symmetric = TRUE,
                                  only.values = TRUE)$values},
               simplify="array")

  FS.E = array(FS.E, dim = c(1, dim(FS.E)))

  laglist = unfold(lapply(L, function(h) {-h:h}))

  filters = fsd.fourier.inverse(FS.E, FS$freq, laglist)

  covs = array(unlist(filters$operators),
               dim = c(dim(FS.E)[2], length(filters$operators)),
               dimnames = list(paste("PC", 1:dim(FS.E)[2]), NULL))

  list(laglist = laglist, cov = covs)
}

#' Calculate the differences for a Functional Spatial Data Object
#'
#' This function is used to calculate the differences of functional data on a
#' spatial grid.
#' @param x the functional spatial data.
#' @param dim the dimension with respect to which the differences are to be
#'   computed.
#' @param lag the lag.
#' @param differences the order of the differences.
#' @param ... currently unused
#' @seealso \code{\link{fsd.fd}}
#' @keywords fsd
#' @export
#' @return
#' An object of class `"fsd.fd"` representing the functional difference
#' between the input functional spatial data objects.
#' @examples
#' data("temp")
#' plot(diff(temp, dim = 1))

diff.fsd.fd = function (x, lag = 1, differences = 1, ..., dim = 1)
{
  if (!inherits(x, "fd"))
    stop("x needs to be functional data")

  r = length(dim(x$coefs)) - 1

  if (dim > r || dim < 1)
    stop("Please select a valid dimension")

  if (dim(x$coefs)[1+dim] <= 1 + differences * lag)
    stop("Not enough data for the computation of the differences")

  pind = c(1+1:dim, 1)
  if (dim < r)
    pind = c(pind,  1+(dim+1):r)

  fdnames = x$fdnames
  if (length(fdnames[[1 + dim]]) > lag)
    fdnames[[1 + dim]] = fdnames[[1 + dim]][-(1:lag)]
  names(fdnames)[1 + dim] = paste0(names(fdnames)[1 + dim],
                                   " (differences of order ", differences,
                                   " with lag ", lag, ")")

  diffcoef = apply(x$coefs, -(1 + dim), diff,
                   lag = lag, differences = differences)

  fsd.fd(coef = aperm(diffcoef, pind),
         basisobj = x$basis, fdnames = fdnames)
}

#' Compute the norm of functional data
#'
#' This function is used to compute the norm of functional data.
#' @param X the functional spatial data.
#' @return an array of the norms.
#' @keywords fsd
#' @export
#' @examples

fsd.norm = function (X)
{
  if (!inherits(X, "fd"))
    warning("X is not a functional data object")

  B = inprod(X$basis, X$basis)
  r = length(dim(X$coefs)) - 1

  sqrt(apply(X$coefs, 1+1:r, function(x) {t(x) %*% B %*% x}))
}

#' Plot method for mfsd.fd objects
#'
#' @param x object of class mfsd.fd
#' @param xlim limits of x axis
#' @param ylim limits of y axis
#' @param main title
#' @param xlab x label
#' @param ylab y label
#' @param gridsize grid size
#' @param set.dims dimension selection
#' @param ... additional graphical parameters
#' @return
#' No return value. This function is called for its side effect of
#' producing graphical plots of the functional spatial data.
#'
#' @method plot mfsd.fd
#' @export



plot.mfsd.fd <- function(x, xlim = NULL, ylim = NULL, main = NULL,
                         xlab = NULL, ylab = NULL,
                         gridsize = NULL, set.dims = NULL, ...) {

  object <- x

  y <- length(object)

  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))

  for (i in 1:y) {
    if (!inherits(object[[i]], "fd")) {
      stop("'object' is not a functional data type.")
    }

    fd_mfsd = vector("list", y)
    oldClass(fd_mfsd ) <- ("fd_mfsd")



  }

  par(mfrow = c(1, y))

  for (i in 1:y) {
    fsd.plot.data(object[[i]], basisobj = NULL, gridsize = gridsize, set.dims = set.dims,
                  xlim = xlim, ylim = ylim, main = main,
                  xlab = xlab, ylab = ylab)
  }
}

#' Perform a Jarque-Bera Test on Normality of Functional Spatial Data
#'
#' This function performs a test on normality of some functional spatial data.
#' @param X.spca a spectral principal components analysis as performed by
#'   \code{\link{fsd.spca}}, i.e. a list that in particular contains the
#'   estimated SPC scores and the estimated spectral density.
#' @param Npc the number of spectral principal components to use for the test.
#' @param L the maximum lag to compute the covariance of the scores during the
#'   computation of the autocovariances.
#' @param var.method the method that is used to calculate the long-run variance
#'   of the SFPC scores. Either "direct" or "integral".
#' @return A list with components \item{T4}{the test statistic.}
#'   \item{p.value}{the p-value.} \item{df}{the degrees of freedom.}
#'   \item{T4.vector}{the vector of the test statistics for each single SPC.}
#' @seealso \code{\link{fsd.spca}}, \code{\link{fsd.spca.cov}}
#' @details To ensure accuracy of numerical integration during the Fourier
#'   transform, the frequencies of \code{FS} should be a suitably dense grid.
#' @keywords fsd
#' @import stats
#' @export


fsd.jb.test = function (X.spca, Npc = NULL, L = NULL, var.method = "integral")
{
  if (is.null(X.spca$FS) && var.method != "direct")
    stop("X.spca needs to contain the spectral density.")
  if (is.null(X.spca$scores))
    stop("X.spca needs to contain the SPC scores.")

  Y = X.spca$scores
  dims = dim(Y)[-1]

  if (is.null(Npc))
    Npc = dim(Y)[1]
  else
    Npc = min(Npc, dim(Y)[1])

  Y = Y - apply(Y, 1, mean)

  mu2 = apply(Y^2, 1, mean)[1:Npc]
  mu3 = apply(Y^3, 1, mean)[1:Npc]
  mu4 = apply(Y^4, 1, mean)[1:Npc]

  if (is.null(L))
    L = dims - 1
  else if (length(L) == 1)
    L = rep(L,length(dims))
  if (var.method == "direct") {
    hlist = unfold(lapply(L, function(h) {-h:h}))
    Clist = sapply(hlist, FUN = fsd.covariance, Y = Y, centered = TRUE,
                   simplify = "array")
    if (is.null(dim(Clist))) {
      C = Clist
      dim(C) = c(1, length(C))
    } else
      C = apply(Clist, 3, diag)
  } else
    C = fsd.spca.cov(FS = X.spca$FS, L = L)$cov

  FS3 = apply(C[1:Npc, , drop = FALSE]^3, 1, sum)
  FS4 = apply(C[1:Npc, , drop = FALSE]^4, 1, sum)

  T4v = prod(dims) * ( mu3^2 / (6 *FS3) + (mu4 - 3*mu2^2)^2 / (24 *FS4)  )

  T4 = sum(T4v)

  list(T4 = T4, p.value = pchisq(T4, df = 2*length(T4v), lower.tail = FALSE),
       df = 2*length(T4v), T4.vector = T4v)
}

#' Calculate the Spectral Principal Components Filters Spatial Functional Data
#'
#' This function calculates the SPC filters from a given spectral density
#' operator.
#' @param FS the spectral density.
#' @param Npc the number of principal components to be computed.
#' @param L the maximum lag for the filters, as an integer or vector of
#'   integers.
#' @return the SPC filters.
#' @seealso \code{\link{fsd.spca}}
#' @details The eigenvectors used for the calculation of the Fourier expansion
#'   are oriented such that the sum of their coordinates over the basis of the
#'   fd object (i.e. the entries in the \code{coefs} array) is a non-negative
#'   real number. If the basis is not orthonormal, this is done with respect to
#'   a virtual orthonormal basis in the background.
#' @keywords fsd
#' @export

fsd.spca.filters = function (FS, Npc = 1, L = 3)
{
  if (!inherits(FS, "fsd.freqdom"))
    stop("FS needs to be a spectral density operator")

  L = floor(L)

  if (length(L) == 1)
    L = rep(L, length(FS$freq[[1]]))

  if (length(L) != length(FS$freq[[1]]))
    stop("Dimensions incompatible")

  dims = dim(FS$operators[[1]])[1]
  Npc = min(Npc, dims)

  if (!is.null(FS$basis)) {
    B = inprod(FS$basis, FS$basis)
  } else {
    B = diag(dim(FS$operators[[1]])[1])
  }
  B.root = eigen(B)$vectors %*% diag(sqrt(eigen(B)$values)) %*%
    t(eigen(B)$vectors)
  B.root.minus = solve(B.root)
  for (i in 1:length(FS$operators)) {
    FS$operators[[i]] = B.root %*% FS$operators[[i]] %*% B.root
  }

  Esys = lapply(FS$operators, eigen, symmetric = TRUE)
  Evecs = sapply(Esys, function(w) {w$vectors[ ,1:Npc, drop=FALSE]},
                 simplify="array")

  Evals = sapply(Esys, '[[', "values", simplify="array")
  Evals.int = apply(pmax(Re(Evals), 0), 1, sum)
  names(Evals.int) = paste("PC", 1:dim(Evals)[1])

  sfpca.var = Evals.int/sum(Evals.int)

  v = rep(1, dims)
  phiv = apply(Evecs, 2:3, "%*%", v)
  phiv[abs(phiv) < 1e-14] = 1
  phiv = phiv/abs(phiv)

  Evecs = Evecs / rep(phiv, each = dims)

  laglist = unfold(lapply(L, function(h) {-h:h}))

  filters = fsd.fourier.inverse(Evecs, FS$freq, laglist)

  for (i in 1:length(filters$operators)) {
    filters$operators[[i]] = B.root.minus %*% filters$operators[[i]]
  }

  filters$basis = FS$basis

  filters$var = sfpca.var

  phinorms = sapply(filters$operators, function(M) {diag(t(M) %*% B %*% M)})

  if (Npc == 1)
    phinorms = sum(phinorms)
  else
    phinorms = apply(phinorms, 1, sum)

  if (any(phinorms < 0.9))
    warning("The chosen clipping parameter L is low.\n",
            "Summed squared norms for the PC filters are only ",
            toString(round(phinorms, 3)))

  filters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.perm.R
\name{fsd.perm}
\alias{fsd.perm}
\title{Permute the dimensions of a Functional Spatial Data Grid}
\usage{
fsd.perm(fsdobj, perm = NULL)
}
\arguments{
\item{fsdobj}{the functional spatial data.}

\item{perm}{the subscript permutation vector, i.e. a permutation of the
integers 1:r.}
}
\value{
A numeric vector containing the permutation indices used to reorder
the functional spatial data.
}
\description{
This function is used to permute the order of the dimensions of the spatial
grid of some functional spatial data.
}
\examples{
data("temp")
fsd.plot.data(fsd.perm(temp, c(2, 1, 3)))
}
\seealso{
\code{\link{fsd.fd}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{as.fd.fsd.fd}
\alias{as.fd.fsd.fd}
\title{Converting a functional spatial data object into a functional data object}
\usage{
\method{as.fd}{fsd.fd}(x, ...)
}
\arguments{
\item{x}{a functional spatial data object}

\item{...}{currently unused}
}
\value{
An object of class \code{fd} (from the \pkg{fda} package).
The returned object corresponds to a functional data object obtained by
collapsing the spatial dimensions of the input.
The coefficients matrix represents functional observations, while the
spatial structure is removed (or flattened).
}
\description{
Works as one would expect
}
\seealso{
\code{\link{fsd.fd}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mfsd_Scores.R
\name{mfsd_Scores}
\alias{mfsd_Scores}
\title{Compute the Multivariate Scores for Spatial Functional Data.}
\usage{
mfsd_Scores(EEM, SM_vec, SM_FSD, X, J)
}
\arguments{
\item{EEM}{concatenated scores.}

\item{SM_vec}{list of variable eigenvectors.}

\item{SM_FSD}{list of univariate spatial functional principal component analysis.}

\item{X}{list of univariate spatial functional data.}

\item{J}{number of functional multivariate spatial principal components.}
}
\value{
a list of the multivariate scores.
}
\description{
Compute the Multivariate Scores for Spatial Functional Data.
}
\keyword{mfsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{/.fsd.fd}
\alias{/.fsd.fd}
\title{Arithmetic for functional spatial data objects}
\usage{
\method{/}{fsd.fd}(fsdobj, scalar)
}
\arguments{
\item{fsdobj}{a functional spatial data object}

\item{scalar}{a scalar number}
}
\value{
An object of class \code{fsd.fd}.
The result corresponds to a scalar division of the functional spatial data
object, where all coefficients are divided by the given scalar.
}
\description{
Works as one would expect.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qlist.R
\docType{data}
\name{qlist}
\alias{qlist}
\title{a list containing tuning parameter for the estimation of the the
spectral density of indian variable}
\format{
qlist has a dimension of 2 22

An object of class \code{"matrix" "array"}.
}
\usage{
data(qlist)
}
\description{
a list containing tuning parameter for the estimation of the the
spectral density of indian variable
}
\examples{
data(qlist)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.plot.covariance.R
\name{fsd.plot.covariance}
\alias{fsd.plot.covariance}
\title{Plot the autocovariance operator of functional spatial data}
\usage{
fsd.plot.covariance(
  X,
  basisobj = NULL,
  q = 0,
  gridsize = NULL,
  set.dims = NULL,
  plot.cor = FALSE,
  na.rm = FALSE,
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{X}{the functional spatial data. Either an fd object or an array.}

\item{basisobj}{the basis of the functional data.}

\item{q}{the maximal lag to be plotted.}

\item{gridsize}{the resolution of the grid on which the kernel will be
evaluated.}

\item{set.dims}{the coordinates to be fixed. A vector of dimension r. NaN
represents not fixed components.}

\item{plot.cor}{a boolean indicating whether the correlation or the
covariance should be plotted.}

\item{na.rm}{whether or not missing values should be ignored.}

\item{main}{title of the plot.}

\item{xlab}{label for the x-axis.}

\item{ylab}{label for the y-axis.}

\item{...}{other arguments to pass to the plot function}
}
\value{
No return value. The function computes the covariance or correlation
operator of the functional spatial data
}
\description{
This function is used to plot the autocovariance operator of functional
spatial data along a two-dimensional plane of lags.
}
\examples{
data("temp")
fsd.plot.covariance(temp, q = 3, plot.cor = TRUE, zlim = c(-1,1))
}
\seealso{
\code{\link{fsd.plot.data}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.mfsd.filter.R
\name{summary.mfsd.filter}
\alias{summary.mfsd.filter}
\title{Analyse multivariate functional spatial filters}
\usage{
\method{summary}{mfsd.filter}(object, use.norm = Inf, make.plot = FALSE)
}
\arguments{
\item{object}{containing a list of variable filters.}

\item{use.norm}{which norm to use for the lags.}

\item{make.plot}{a boolean indicating whether to plot the result.}
}
\value{
object list with components \item{balls}{a data frame containing the
  cumulative sums of the squared norms of the filter coefficients up to some
  maximum lag.} \item{zeroplanes}{a matrix containing the sums of the squared
  norms of the filter coefficients lying on the (hyper-) planes with one
  dimension of the lag equals zero, along with the sum over all lags as
  comparison.} \item{abs.sum}{a data frame containing the cumulative sums of
  the norms of the filter coefficients.}
}
\description{
This function is used to analyse multivariate functional spatial filters, indicating how
the squared norms lie.
}
\keyword{mfsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.jb.test.R
\name{fsd.jb.test}
\alias{fsd.jb.test}
\title{Perform a Jarque-Bera Test on Normality of Functional Spatial Data}
\usage{
fsd.jb.test(X.spca, Npc = NULL, L = NULL, var.method = "integral")
}
\arguments{
\item{X.spca}{a spectral principal components analysis as performed by
\code{\link{fsd.spca}}, i.e. a list that in particular contains the
estimated SPC scores and the estimated spectral density.}

\item{Npc}{the number of spectral principal components to use for the test.}

\item{L}{the maximum lag to compute the covariance of the scores during the
computation of the autocovariances.}

\item{var.method}{the method that is used to calculate the long-run variance
of the SFPC scores. Either "direct" or "integral".}
}
\value{
A list with components \item{T4}{the test statistic.}
  \item{p.value}{the p-value.} \item{df}{the degrees of freedom.}
  \item{T4.vector}{the vector of the test statistics for each single SPC.}
}
\description{
This function performs a test on normality of some functional spatial data.
}
\details{
To ensure accuracy of numerical integration during the Fourier
  transform, the frequencies of \code{FS} should be a suitably dense grid.
}
\seealso{
\code{\link{fsd.spca}}, \code{\link{fsd.spca.cov}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unfold.R
\name{unfold}
\alias{unfold}
\title{Unfold indices into a high-dimensional grid}
\usage{
unfold(indexlist, r = NULL, return.array = FALSE)
}
\arguments{
\item{indexlist}{indices to be expanded. Either a list with each member
representing a dimension, or a vector.}

\item{r}{the dimension of the grid to be created. This is only needed if
indexlist is a vector.}

\item{return.array}{a boolean indicating whether an array should be returned
or a list.}
}
\value{
a list or an array with the coordinates of the grid.
}
\description{
This function is used to expand indices into a grid. Works similar to
\code{\link[base]{expand.grid}}.
}
\examples{
unfold(list(11:12, 21:23), return.array = TRUE)
}
\keyword{mfsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fourier.inverse.R
\name{fsd.fourier.inverse}
\alias{fsd.fourier.inverse}
\title{Compute the Fourier inverse}
\usage{
fsd.fourier.inverse(evecs, thlist, laglist)
}
\arguments{
\item{evecs}{the eigenvectors to be expanded.}

\item{thlist}{list of frequencies.}

\item{laglist}{list of lags.}
}
\value{
the filter obtained by applying the Fourier inverse on evecs.
}
\description{
This function is used to calculate the Fourier inverse.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{[.fsd.fd}
\alias{[.fsd.fd}
\title{Subsetting a functional spatial data object}
\usage{
\method{[}{fsd.fd}(fsdobj, ..., drop = TRUE)
}
\arguments{
\item{fsdobj}{a functional spatial data object}

\item{...}{the indices to take}

\item{drop}{whether to flatten the spatial grid by dropping dimensions of
length one.}
}
\value{
An object of class \code{fsd.fd}.
The returned object is a subset of the original functional spatial data object.
The structure of the coefficient array and the associated spatial grid is
reduced according to the selected indices. If \code{drop = TRUE}, dimensions
of length one in the spatial grid may be removed.
}
\description{
Subsetting works as with conventional arrays. By default, dimensions can be
dropped.
}
\examples{
data("temp")
plot(temp[1:2, 1:2, 1, drop = TRUE])
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{fsd.fd}
\alias{fsd.fd}
\title{Creating a functional spatial data object}
\usage{
fsd.fd(coef = NULL, basisobj = NULL, fdnames = NULL)
}
\arguments{
\item{coef}{an array of arbitrary dimension 1 + r, where the first dimension
corresponds to basis functions.}

\item{basisobj}{a functional basis object defining the basis.}

\item{fdnames}{a list of length r with each member containing the labels for
the dimensions of the data.}
}
\value{
the \code{fsd.fd} object.
}
\description{
This function extends the possibilities of the function fd found in the
package fda to include functional data on a spatial grid of arbitrary
dimension r.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.spca.filters.R
\name{fsd.spca.filters}
\alias{fsd.spca.filters}
\title{Calculate the Spectral Principal Components Filters Spatial Functional Data}
\usage{
fsd.spca.filters(FS, Npc = 1, L = 3)
}
\arguments{
\item{FS}{the spectral density.}

\item{Npc}{the number of principal components to be computed.}

\item{L}{the maximum lag for the filters, as an integer or vector of
integers.}
}
\value{
the SPC filters.
}
\description{
This function calculates the SPC filters from a given spectral density
operator.
}
\details{
The eigenvectors used for the calculation of the Fourier expansion
  are oriented such that the sum of their coordinates over the basis of the
  fd object (i.e. the entries in the \code{coefs} array) is a non-negative
  real number. If the basis is not orthonormal, this is done with respect to
  a virtual orthonormal basis in the background.
}
\seealso{
\code{\link{fsd.spca}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{-.fsd.fd}
\alias{-.fsd.fd}
\title{Arithmetic for functional spatial data objects}
\usage{
\method{-}{fsd.fd}(fsd1, fsd2 = NULL)
}
\arguments{
\item{fsd1, fsd2}{functional spatial data objects}
}
\value{
An object of class \code{fsd.fd}.
If a single argument is provided, the result is the additive inverse of the
input object. Otherwise, it corresponds to the element-wise difference between
two functional spatial data objects.
}
\description{
Works as one would expect.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{as.fsd.fd}
\alias{as.fsd.fd}
\title{Converting a functional data object into a functional spatial data object}
\usage{
as.fsd.fd(x, ...)
}
\arguments{
\item{x}{a functional data object or something that can be converted into
one.}

\item{...}{currently unused}
}
\value{
An object of class \code{fsd.fd}.
The returned object represents functional data indexed over a spatial grid.
It contains: \code{coefs}: an array of coefficients where the first dimension
 corresponds to basis coefficients and the remaining dimensions correspond
 to spatial locations,
\code{basis}: a basis object describing the functional representation,
\code{fdnames}: a list describing the axes (spatial indices and function domain).
This transformation embeds classical functional data into a spatial framework.
}
\description{
Works as one would expect
}
\seealso{
\code{\link{fsd.fd}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.mfsd.fd.R
\name{plot.mfsd.fd}
\alias{plot.mfsd.fd}
\title{Plot method for mfsd.fd objects}
\usage{
\method{plot}{mfsd.fd}(
  x,
  xlim = NULL,
  ylim = NULL,
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  gridsize = NULL,
  set.dims = NULL,
  ...
)
}
\arguments{
\item{x}{object of class mfsd.fd}

\item{xlim}{limits of x axis}

\item{ylim}{limits of y axis}

\item{main}{title}

\item{xlab}{x label}

\item{ylab}{y label}

\item{gridsize}{grid size}

\item{set.dims}{dimension selection}

\item{...}{additional graphical parameters}
}
\value{
No return value. This function is called for its side effect of
producing graphical plots of the functional spatial data.
}
\description{
Plot method for mfsd.fd objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Llist.R
\docType{data}
\name{Llist}
\alias{Llist}
\title{a list containing the maximum lag for the filters for each
variable.}
\format{
Llist has a dimension of [1:22]

An object of class \code{int [1:22]}.
}
\usage{
data(Llist)
}
\description{
an example taken from the variable indian which represents a list
containing the maximum lag for the filters for each variable.
An integer or vector of #' integers.
}
\examples{
data(Llist)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{\%/\%.fsd.fd}
\alias{\%/\%.fsd.fd}
\title{Subsetting a functional spatial data object}
\usage{
\method{\%/\%}{fsd.fd}(fsdobj, s)
}
\arguments{
\item{fsdobj}{a functional spatial data object}

\item{s}{an integer or a vector with one entry for every dimension of the
spatial grid.}
}
\value{
An object of class \code{fsd.fd}.
The returned object is a downsampled version of the input functional spatial
data object, where only every \code{s}-th spatial location is retained along
each spatial dimension.
}
\description{
Take only every \code{s}-th element
}
\examples{
data("temp")
plot(temp \%/\% 3)
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.sfarma.R
\name{fsd.sfarma}
\alias{fsd.sfarma}
\title{Simulate a spatial functional ARMA process}
\usage{
fsd.sfarma(
  n,
  Sigma = NULL,
  ARfilter = NULL,
  MAfilter = NULL,
  burnin = 30,
  basisobj = NULL,
  noise = "normal",
  do.fixed.point = FALSE,
  max.iter = 50,
  eps = 1e-05
)
}
\arguments{
\item{n}{the sample size.}

\item{Sigma}{the covariance matrix of the noise.}

\item{ARfilter}{the functional spatial filter of the autoregressive part.}

\item{MAfilter}{the functional spatial filter of the moving-average part.}

\item{burnin}{the number of grid points to add on each side of the sample to
achieve stationarity.}

\item{basisobj}{the basis of the functional data.}

\item{noise}{the noise distribution to use. Either "normal" or an integer for
the degrees of freedom of a multivariate Student's t-distribution.}

\item{do.fixed.point}{whether to always use a fixed point iteration.}

\item{max.iter}{the maximum number of iterations in the fixed point
iteration.}

\item{eps}{the stopping criterion for the fixed point iteration.}
}
\value{
one sample of the SFARMA process.
}
\description{
This function is used to simulate a sample from a spatial functional ARMA
(SFARMA) process using a normal or t-distributed noise process.
}
\details{
Please note that convergence in the fixed point iteration depends strongly on
the sample size and the structure of the AR filter. For certain choices of
parameters, even more than \code{n} iterations may be needed.
}
\seealso{
\code{\link{fsd.fd}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.spca.scores.R
\name{fsd.spca.scores}
\alias{fsd.spca.scores}
\title{Compute the Scores for Spatial Functional Data}
\usage{
fsd.spca.scores(X, A, na.ignore = FALSE)
}
\arguments{
\item{X}{the functional spatial data. Either an fd object or an array.}

\item{A}{the filter used to compute the scores.}

\item{na.ignore}{whether missing values in the data X should be ignored.}
}
\value{
the array of the scores.
}
\description{
This function computes the scores. It applies the matrices of the filter
\code{A} to the data \code{X}.
}
\seealso{
\code{\link{fsd.spca.inverse}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{temp}
\alias{temp}
\title{Temperature Data in Wyoming}
\format{
An object of class \code{"fsd.fd"}.
}
\usage{
data(temp)
}
\description{
CPC daily Temperature data of mean temperatures in Wyoming
projected onto a B-spline basis of 36 functions for each year,
from 1979 to 2017 and with a spatial resolution of 0.5 degrees.
}
\details{
The gridded data has been obtained by Shepard's method.
}
\examples{
data(temp)
plot(temp)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.plot.filters.R
\name{fsd.plot.filters}
\alias{fsd.plot.filters}
\title{Plot functional spatial filters}
\usage{
fsd.plot.filters(
  A,
  basisobj = NULL,
  gridsize = NULL,
  Npc = 3,
  Lmax = 3,
  set.dims = NULL,
  main = NULL,
  xlab = NULL,
  ylab = NULL
)
}
\arguments{
\item{A}{the filters.}

\item{basisobj}{the basis of the functional data.}

\item{gridsize}{the number of grid points on which to evaluate the functional
data.}

\item{Npc}{number of filters to plot.}

\item{Lmax}{maximum lag to plot.}

\item{set.dims}{the coordinates to be fixed. A vector of dimension r. NaN
represents not fixed components.}

\item{main}{title of the plot.}

\item{xlab}{label for the x-axis.}

\item{ylab}{label for the y-axis.}
}
\value{
No return value. This function produces a graphical representation
of functional spatial filters.
}
\description{
This function is used to plot functional spatial filters,  along a
two-dimensional plane of indices.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/center.fsd.fd.R
\name{center.fsd.fd}
\alias{center.fsd.fd}
\title{Compute the centered version of spatial functional data}
\usage{
center.fsd.fd(X, margins = NULL, na.rm = FALSE)
}
\arguments{
\item{X}{The functional spatial data.}

\item{margins}{The margins for which the centering should be done separately
in every entry, in order to remove a trend.}

\item{na.rm}{Whether or not missing values should be ignored.}
}
\value{
The centered data.
}
\description{
This function is used to center spatial functional data.
}
\examples{
data("temp")

T.centered = center.fsd.fd(temp)
plot(mean(T.centered, margins = 1:2))

T.centered2 = center.fsd.fd(temp, margins = 1:2)
plot(mean(T.centered2, margins = 1:2))
plot(mean(temp, margins = 1:2))
}
\seealso{
\code{\link{mean.fsd.fd}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mfsd_Filter.R
\name{mfsd_Filter}
\alias{mfsd_Filter}
\title{Calculate the Multivariate Spectral Principal Components Filters Spatial Functional Data}
\usage{
mfsd_Filter(SM_FSD, SM_vec, J)
}
\arguments{
\item{SM_FSD}{list of univariate spatial functional principal component analysis.}

\item{SM_vec}{list of variable eigenvectors.}

\item{J}{number of functional multivariate spatial principal components.}
}
\value{
the Multivariate SPC filters.
}
\description{
This function calculates the multivariate SPC filters from a given spectral density operator.
}
\details{
The univariate eigenvectors and filters are used to compute multivariate spatial function filters.
}
\keyword{mfsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fourier.R
\name{fsd.fourier}
\alias{fsd.fourier}
\title{Compute the Fourier transform}
\usage{
fsd.fourier(covs, laglist, thlist)
}
\arguments{
\item{covs}{the covariances to be transformed}

\item{laglist}{list of lags.}

\item{thlist}{list of frequencies.}
}
\value{
the Fourier transform of the covariances.
}
\description{
This function is used to calculate the Fourier transform
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diff.fsd.fd.R
\name{diff.fsd.fd}
\alias{diff.fsd.fd}
\title{Calculate the differences for a Functional Spatial Data Object}
\usage{
\method{diff}{fsd.fd}(x, lag = 1, differences = 1, ..., dim = 1)
}
\arguments{
\item{x}{the functional spatial data.}

\item{lag}{the lag.}

\item{differences}{the order of the differences.}

\item{...}{currently unused}

\item{dim}{the dimension with respect to which the differences are to be
computed.}
}
\value{
An object of class `"fsd.fd"` representing the functional difference
between the input functional spatial data objects.
}
\description{
This function is used to calculate the differences of functional data on a
spatial grid.
}
\examples{
data("temp")
plot(diff(temp, dim = 1))
}
\seealso{
\code{\link{fsd.fd}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.spca.cov.R
\name{fsd.spca.cov}
\alias{fsd.spca.cov}
\title{Calculate the Covariance of the Spectral Principal Component Scores}
\usage{
fsd.spca.cov(FS, L = 3)
}
\arguments{
\item{FS}{the spectral density.}

\item{L}{the maximum lag to compute the covariance of the scores}
}
\value{
A list with components \item{laglist}{the list of lags.} \item{cov}{a
  matrix with the autocovariances of the principal component scores.}
}
\description{
This function estimates the (theoretical) autocovariance for each spectral
principal component score for some spatial functional data.
}
\details{
To ensure accuracy of numerical integration during the Fourier
  transform, the frequencies of \code{F} should be a suitably dense grid.
}
\seealso{
\code{\link{fsd.spca}}, \code{\link{fsd.spca.var}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.norm.R
\name{fsd.norm}
\alias{fsd.norm}
\title{Compute the norm of functional data}
\usage{
fsd.norm(X)
}
\arguments{
\item{X}{the functional spatial data.}
}
\value{
an array of the norms.
}
\description{
This function is used to compute the norm of functional data.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mean.fsd.fd.R
\name{mean.fsd.fd}
\alias{mean.fsd.fd}
\title{Compute the mean of spatial functional data}
\usage{
\method{mean}{fsd.fd}(x, trim = 0, na.rm = FALSE, ..., margins = NULL)
}
\arguments{
\item{x}{the functional spatial data.}

\item{trim}{currently unused}

\item{na.rm}{whether or not missing values should be ignored.}

\item{...}{currently unused}

\item{margins}{the margins that should not be averaged over.}
}
\value{
the mean of x
}
\description{
This function is used to calculate the mean on functional data on a spatial
grid.
}
\examples{
data("temp")
plot(mean(temp))
plot(mean(temp, margins = 1:2))
}
\seealso{
\code{\link{center.fsd.fd}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mfsd.R
\name{mfsd}
\alias{mfsd}
\title{Perform Multivariate Spectral Principal Components Analysis on Spatial Functional Data.}
\usage{
mfsd(
  X,
  freq.res = 100,
  SM_Npc = 3,
  L = 3,
  q = NULL,
  na.ignore = TRUE,
  return.F = FALSE,
  only.filters = FALSE
)
}
\arguments{
\item{X}{a list containing the functional spatial data. Either an fd object or an array.}

\item{freq.res}{the resolution for the computation of the spectral density.}

\item{SM_Npc}{the number of multivariat spatial functional principal components to be computed.}

\item{L}{a list containing the maximum lag for the filters for each variable. An integer or vector of integers.}

\item{q}{a list containing tuning parameter for the estimation of the the spectral density
operator for each variable. An integer or vector of integers.}

\item{na.ignore}{whether to ignore missing data points in the computation of
the scores.}

\item{return.F}{a boolean indicating whether to return the spectral density.}

\item{only.filters}{a boolean indicating whether to compute only the filters,
leaving out the scores.}
}
\value{
A list with components \item{Value}{eigenvalue for each multivariate PC}
  \item{tuning.params}{a list of the used tuning parameters \code{freq.res},
  \code{q} and \code{Lmax}.} \item{filters}{the multivariate SPC filters.}
  \item{scores}{the multivariate SPC scores.} \item{vm}{the theoretical fractions of
  variance explained by each multivariate PC.} \item{ SML_Mean }{the mean of X.}
  \item{ SM_FSD }{list of univariate spatial functional principal component analysis.}
}
\description{
This function performs multivariate PCA on functional spatial data on a grid of
dimension r.
}
\details{
This function can be used to compute spectral PCA. By setting
  \code{q = 0}, it can also be used to perform static MFPCA.

  Setting a higher value for \code{freq.res} increases the runtime   significantly, but yields a more
  accurate result because of reduced integration errors.
}
\examples{
\donttest{
data("X1999")
data("X2000")
data("qlist")
data("Llist")
X = list(X1999, X2000)
q = list(qlist[,17], qlist[,18])
L = list(L1 = Llist[17], L2 = Llist[18])
mfsd(X = X, L = L, q = q, SM_Npc = 4)
}
}
\keyword{mfsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.spectral.density.R
\name{fsd.spectral.density}
\alias{fsd.spectral.density}
\title{Estimate the spectral density operator}
\usage{
fsd.spectral.density(X, freq, q = NULL, na.rm = FALSE)
}
\arguments{
\item{X}{the functional spatial data. Either an fd object or an array.}

\item{freq}{the spatial frequencies with respect to which the spectral
density is to be estimated. A vector or a list of vectors defining the
grid.}

\item{q}{the size of the window of the kernel used for estimation. An integer
or a vector. For any dimension, the value \code{q = 1} is equivalent to
\code{q = 0} and means that no lags in this direction are taken into
account.}

\item{na.rm}{whether or not missing values should be ignored.}
}
\value{
the spectral density, i.e. a list consisting of \item{operators}{a
  list of the the spectral density operator at different spatial
  frequencies.} \item{freq}{a list of the spatial frequencies.}
  \item{basis}{the basis object of the functional data.}
  \item{estimation.q}{the tuning parameter \code{q} used in the estimation
  process.}
}
\description{
This function is used to estimate the spectral density operator of stationary
functional spatial data on a spatial frequency grid.
}
\details{
The spectral density is estimated employing a  Bartlett kernel on
  the Euclidean norm of the lags. Therefore, the lag \code{q} is the smallest
  lag not to be taken into consideration.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mfsd.evaluation.R
\name{mfsd.evaluation}
\alias{mfsd.evaluation}
\title{Perform A Comparative Evaluation of Dimension Reduction: SMFPCA versus MFPCA in Multivariate Spatial Data Reconstruction}
\usage{
mfsd.evaluation(Xspec, Xspecc, X00, Npc = 3)
}
\arguments{
\item{Xspec}{an object of class SM_object, this function is obtained by applying the mfsd
function with a non-zero value of q.}

\item{Xspecc}{an object of class M_object, this object is obtained by applying the mfsd
function with q equal to zero.}

\item{X00}{a list containing the functional spatial data. Either an fd object or an array.}

\item{Npc}{the number of multivariat spatial functional principal components to be computed.}
}
\value{
a display of the NMSE and NMSE* results from \code{\link{print}}.
}
\description{
To assess the effectiveness of integrating the spatial and multivariate aspects through SMFPCA, it is
common to reconstruct the original functional data from the already computed scores and filters.
For each instance, we employ both the novel SMFPCA and the conventional MFPCA. We assess the
effectiveness of dimension reduction using the metric known as the normalized mean squared
error (NMSE) for each univariate component. NMSE* evaluates the quality of the approximation without being
influenced by grid boundary effects, unlike NMSE, where the approximation is
less accurate at the boundary.
}
\keyword{mfsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.filter.R
\name{fsd.filter}
\alias{fsd.filter}
\title{Creating a functional spatial filter}
\usage{
fsd.filter(laglist, operatorlist, basisobj = NULL)
}
\arguments{
\item{laglist}{a list of lags.}

\item{operatorlist}{a list of operators, i.e. matrices.}

\item{basisobj}{a functional basis object defining the basis.}
}
\value{
the \code{fsd.filter} object.
}
\description{
This function creates a functional spatial filter
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.filter.is.unilateral.R
\name{fsd.filter.is.unilateral}
\alias{fsd.filter.is.unilateral}
\title{Check if functional spatial filter is unilateral}
\usage{
fsd.filter.is.unilateral(A, exclude.origin = FALSE)
}
\arguments{
\item{A}{the functional spatial filter.}

\item{exclude.origin}{whether to exclude the origin from the definition of
unilaterality.}
}
\value{
a Boolean indicating whether A is unilateral or not.
}
\description{
This function is used to verify if a functional spatial filter is unilateral.
}
\seealso{
\code{\link{fsd.filter}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.spca.inverse.R
\name{fsd.spca.inverse}
\alias{fsd.spca.inverse}
\title{Reconstruct the original functional data from the filters and the score}
\usage{
fsd.spca.inverse(A, scores, mean.X = NULL)
}
\arguments{
\item{A}{the filters.}

\item{scores}{the scores as an array.}

\item{mean.X}{the mean of X.}
}
\value{
X the reconstructed functional data
}
\description{
This function is used to reconstruct the original functional data from the
filters and the score. It applies the transposed matrices of the filter
\code{A} to the scores.
}
\seealso{
\code{\link{fsd.spca}}, \code{\link{fsd.spca.scores}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.spca.R
\name{fsd.spca}
\alias{fsd.spca}
\title{Perform Spectral Principal Components Analysis on Spatial Functional Data}
\usage{
fsd.spca(
  X,
  freq.res = 100,
  Npc = 3,
  L = 3,
  q = NULL,
  na.ignore = TRUE,
  return.F = FALSE,
  only.filters = FALSE
)
}
\arguments{
\item{X}{the functional spatial data. Either an fd object or an array.}

\item{freq.res}{the resolution for the computation of the spectral density.}

\item{Npc}{the number of principal components to be computed.}

\item{L}{the maximum lag for the filters. An integer or vector of integers.}

\item{q}{a tuning parameter for the estimation of the the spectral density
operator. An integer or vector of integers.}

\item{na.ignore}{whether to ignore missing data points in the computation of
the scores.}

\item{return.F}{a boolean indicating whether to return the spectral density.}

\item{only.filters}{a boolean indicating whether to compute only the filters,
leaving out the scores.}
}
\value{
A list with components \item{F}{the spectral density.}
  \item{tuning.params}{a list of the used tuning parameters \code{freq.res},
  \code{q} and \code{Lmax}.} \item{filters}{the SPC filters.}
  \item{scores}{the SPC scores.} \item{var}{the theoretical fractions of
  variance explained by each PC.} \item{X.mean}{the mean of X.}
}
\description{
This function performs spectral PCA on functional spatial data on a grid of
dimension r.
}
\details{
This function can be used to compute spectral PCA. By setting
  \code{q = 0}, it can also be used to perform static PCA.

  Setting a higher value for \code{freq.res} increases the runtime
  significantly, but yields a more accurate result because of reduced
  integration errors.
}
\seealso{
\code{\link{fsd.spca.inverse}}, \code{\link{fsd.spectral.density}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.fsd.filter.R
\name{plot.fsd.filter}
\alias{plot.fsd.filter}
\title{Plot functional spatial filters}
\usage{
\method{plot}{fsd.filter}(
  x,
  xlim = NULL,
  ylim = NULL,
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  gridsize = NULL,
  Npc = 3,
  Lmax = 3,
  set.dims = NULL,
  ...
)
}
\arguments{
\item{x}{the filter object.}

\item{xlim}{limits of the x-axis.}

\item{ylim}{limits of the y-axis.}

\item{main}{main title of the graph.}

\item{xlab}{label of the x-axis.}

\item{ylab}{label of the y-axis.}

\item{gridsize}{number of grid points used to evaluate the functional data.}

\item{Npc}{number of filters to plot.}

\item{Lmax}{maximum lag to plot.}

\item{set.dims}{coordinates to be fixed. A vector of dimension r. `NaN`
represents components that are not fixed.}

\item{...}{additional graphical parameters.}
}
\value{
No return value. This function is called for its side effect of
producing graphical displays of functional spatial filters.
}
\description{
This function plots functional spatial filters along a
two-dimensional plane of indices.
}
\seealso{
\code{\link{fsd.plot.filters}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.plot.data.R
\name{fsd.plot.data}
\alias{fsd.plot.data}
\title{Plot functional spatial data}
\usage{
fsd.plot.data(
  X,
  basisobj = NULL,
  gridsize = NULL,
  set.dims = NULL,
  xlim = NULL,
  ylim = NULL,
  main = NULL,
  xlab = NULL,
  ylab = NULL
)
}
\arguments{
\item{X}{the functional spatial data. Either an fd object or an array, or a
list thereof.}

\item{basisobj}{the basis of the functional data.}

\item{gridsize}{the number of grid points on which to evaluate the functional
data.}

\item{set.dims}{the coordinates to be fixed. A vector of dimension r. NaN
represents not fixed components.}

\item{xlim}{limits for the coordinates on the x-axis.}

\item{ylim}{limits for the coordinates on the y-axis.}

\item{main}{title of the plot.}

\item{xlab}{label for the x-axis.}

\item{ylab}{label for the y-axis.}
}
\value{
No return value. This function produces a graphical representation
of functional spatial data.
}
\description{
This function is used to plot functional spatial data on a grid of dimension
r,  along a two-dimensional plane of indices.
}
\examples{
data("temp")
fsd.plot.data(temp, set.dims = c(NaN, NaN, 3))
}
\seealso{
\code{\link{plot.fsd.fd}}, \code{\link{fsd.plot.filters}},
  \code{\link{fsd.plot.covariance}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.fsd.fd.R
\name{plot.fsd.fd}
\alias{plot.fsd.fd}
\title{Plot functional spatial data}
\usage{
\method{plot}{fsd.fd}(x, ..., gridsize = NULL, set.dims = NULL)
}
\arguments{
\item{x}{the functional spatial data.}

\item{...}{graphical parameters.}

\item{gridsize}{the number of grid points on which to evaluate the functional
data.}

\item{set.dims}{the coordinates to be fixed. A vector of dimension r. NaN
represents not fixed components.}
}
\value{
No return value. This function is called for its side effect of
producing graphical displays of functional spatial data.
}
\description{
This function is used to plot functional spatial data on a grid of dimension
r,  along a two-dimensional plane of indices.
}
\examples{
data("temp")
plot(temp)
}
\seealso{
\code{\link{fsd.plot.data}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{*.fsd.fd}
\alias{*.fsd.fd}
\title{Arithmetic for functional spatial data objects}
\usage{
\method{*}{fsd.fd}(scalar, fsdobj)
}
\arguments{
\item{scalar}{a scalar number}

\item{fsdobj}{a functional spatial data object}
}
\value{
An object of class \code{fsd.fd}.
The result corresponds to a scalar multiplication of the functional spatial
data object, where all coefficients are multiplied by the given scalar.
}
\description{
Works as one would expect.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indian_2000.R
\docType{data}
\name{X2000}
\alias{X2000}
\title{Spatial functional data of sea surface temperature (SST) data
for the year 2000, derived from NOAA optimal sea surface
temperature interpolation data.}
\format{
An object of class \code{"fsd.fd"}.
}
\usage{
data(X2000)
}
\description{
Spatial Functional Data of sea surface temperature (SST) data from the NOAA Optimum
Interpolation Sea Surface Temperature dataset.
The dataset exhibiting typical spatial dependence characterized
by an exponential decay.
}
\examples{
data(X2000)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.covariance.R
\name{fsd.covariance}
\alias{fsd.covariance}
\title{Estimate autocovariance operators}
\usage{
fsd.covariance(Y = NULL, h, centered = FALSE, na.rm = FALSE, unbiased = FALSE)
}
\arguments{
\item{Y}{the functional spatial data. Either an fd object or an array.}

\item{h}{the spatial lag with respect to which the autocovariance is to be
estimated.}

\item{centered}{a boolean indicating whether or not the data is already
centered.}

\item{na.rm}{whether or not missing values should be ignored.}

\item{unbiased}{whether or not to scale the estimator to be unbiased. FALSE
by default. Is not used if na.rm is TRUE.}
}
\value{
the covariance matrix.
}
\description{
This function is used to estimate a covariance operator of stationary
functional spatial data with respect to some lag h.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.spca.var.R
\name{fsd.spca.var}
\alias{fsd.spca.var}
\title{Calculate the Variance explained by Spectral Principal Components}
\usage{
fsd.spca.var(FS)
}
\arguments{
\item{FS}{the spectral density.}
}
\value{
the fraction of the variance explained by each PC.
}
\description{
This function estimates the (theoretical) fraction of the variance explained
by each spectral principal component for some spatial functional data.
}
\details{
To ensure accuracy of numerical integration, the frequencies of
  \code{FS} should be a suitably dense grid. In order for the estimations to
  conform with the actual variance explained (1 - NMSE), the tuning
  parameters in the estimation of the spectral density need to be chosen
  carefully.
}
\seealso{
\code{\link{fsd.spca}}
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.z.plot.R
\name{fsd.z.plot}
\alias{fsd.z.plot}
\title{Plot functional data on a grid}
\usage{
fsd.z.plot(Zlist, positions, basisobj = NULL, gridsize = 50, ...)
}
\arguments{
\item{Zlist}{list of arrays with coefficients.}

\item{positions}{the positions of the single curves in space.}

\item{basisobj}{the basis of the functional data.}

\item{gridsize}{the number of grid points on which to evaluate the curves.}

\item{...}{other arguments to pass to the plot function}
}
\value{
No return value. This function is called for its side effect of
producing a graphical representation of functional data evaluated
on a spatial grid.
}
\description{
This function plots functional data on a grid.
}
\keyword{fsd}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsd.fd.R
\name{+.fsd.fd}
\alias{+.fsd.fd}
\title{Arithmetic for functional spatial data objects}
\usage{
\method{+}{fsd.fd}(fsd1, fsd2)
}
\arguments{
\item{fsd1, fsd2}{functional spatial data objects}
}
\value{
An object of class \code{fsd.fd}.
The result corresponds to the element-wise sum of the coefficient arrays of
the two functional spatial data objects. The spatial structure and basis are preserved.
}
\description{
Works as one would expect.
}
\keyword{fsd}
